#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "xpu2image4xi.py">
#<abstract4xi abstract = "XPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "xpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'XPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4xpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4xpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4xpu',              # ACK 5.3.4 p92
            'create2image4xpu',                    # ACK 5.31  p76            

            'get2imageinfo4xpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4xpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4xpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4xpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4xpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4xpu',     # ACK modernized
            'manoeuvre2mapimage4xpu',              # ACK modernized                                           
            'manoeuvre2readimage4xpu',             # ACK modernized
            'manoeuvre2writeimage4xpu',            # ACK modernized            
            'map2image4xpu',                       # ACK 5.3.5 p94 
            
            'read2image4xpu',                      # ACK 5.3.3 p84  
            'retain2image4xpu',                    # ACK 5.4.1 p99
            'release2image4xpu',                   # ACK 5.4.1 p99

            'set2imagecallback4xpu',               # ACK modernized
            'set2image4xpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4xpu',                     # ACK modernized 
                                    
            'wait2image4xpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4xpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from xpu2buffer4xi import XPU2Buffer4Xi
from xpu2buffer4xi import create2buffer4xpu
from xpu2buffer4xi import create2subbuffer4xpu
from xpu2buffer4xi import get2bufferinfo4xpu
from xpu2buffer4xi import release2buffer4xpu
from xpu2buffer4xi import retain2buffer4xpu
from xpu2buffer4xi import set2buffer4xpu
from xpu2buffer4xi import trace2buffer4xpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from xpu2commandqueue4xi import XPU2CommandQueue4Xi
from xpu2commandqueue4xi import create2commandqueue4xpu 
from xpu2commandqueue4xi import flush2commandqueue4xpu
from xpu2commandqueue4xi import finish2commandqueue4xpu
from xpu2commandqueue4xi import get2commandqueueinfo4xpu
from xpu2commandqueue4xi import release2commandqueue4xpu
from xpu2commandqueue4xi import retain2commandqueue4xpu
from xpu2commandqueue4xi import set2commandqueue4xpu
from xpu2commandqueue4xi import trace2commandqueue4xpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from xpu2context4xi import XPU2Context4Xi 
from xpu2context4xi import create2context4xpu
from xpu2context4xi import get2context4xpu
from xpu2context4xi import release2context4xpu
from xpu2context4xi import retain2context4xpu
from xpu2context4xi import set2context4xpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from xpu2device4xi import XPU4Xi
from xpu2device4xi import init2xpu4xi 
from xpu2device4xi import set2xpu4xi
from xpu2device4xi import get2xpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from xpu2event4xi import XPU2Event4Xi
from xpu2event4xi import create2event4xpu
from xpu2event4xi import get2eventinfo4xpu
from xpu2event4xi import release2event4xpu
from xpu2event4xi import retain2event4xpu
from xpu2event4xi import set2eventcallback4xpu 
from xpu2event4xi import set2event4xpu
from xpu2event4xi import trace2event4xpu
from xpu2event4xi import wait2event4xpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global xpu2image4xi                                                     # 2D-IMAGES FOR XPU

global xpu2trace4xi                                                     # TRACE

global xpu2buffer4xi                                                    # BUFFER FOR XPU

global control2readimage4xpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH XPU  
global state2read4xpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4xpu                                                # MAX FOR STATE
global ack2readinfo4xpu                                                 # ACK FOR IMAGE INFO
global message2readget4xpu                                              # QUEUE GET
global message2readput4xpu                                              # QUEUE PUT
global message2readreceive4xpu                                          # PIPE RECEIVE
global message2readsend4xpu                                             # PIPE SEND

global control2writeimage4xpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH XPU 
global state2write4xpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4xpu                                               # MAX FOR STATE
global ack2writeinfo4xpu                                                # ACK FOR IMAGE INFO
global message2writeget4xpu                                             # QUEUE GET
global message2writeput4xpu                                             # QUEUE PUT
global message2writereceive4xpu                                         # PIPE RECEIVE
global message2writesend4xpu                                            # PIPE SEND

global control2copyimage4xpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH XPU 
global state2copyimage4xpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4xpu                                           # MAX FOR STATE
global ack2copyimageinfo4xpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4xpu                                         # QUEUE GET
global message2copyimageput4xpu                                         # QUEUE PUT
global message2copyimagereceive4xpu                                     # PIPE RECEIVE
global message2copyimagesend4xpu                                        # PIPE SEND

global control2copyimagetobuffer4xpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH XPU 
global state2copyimagetobuffer4xpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4xpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4xpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4xpu                                 # QUEUE GET
global message2copyimagetobufferput4xpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4xpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4xpu                                # PIPE SEND

global control2copybuffertoimage4xpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH XPU 
global state2copybuffertoimage4xpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4xpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4xpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4xpu                                 # QUEUE GET
global message2copybuffertoimageput4xpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4xpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4xpu                                # PIPE SEND                      

global control2mapimage4xpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH XPU  
global state2map4xpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4xpu                                                # MAX FOR STATE
global ack2mapinfo4xpu                                                 # ACK FOR IMAGE INFO
global message2mapget4xpu                                              # QUEUE GET
global message2mapput4xpu                                              # QUEUE PUT
global message2mapreceive4xpu                                          # PIPE RECEIVE
global message2mapsend4xpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4xpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF XPU2IMAGE4XI5TEST - TRYING TO TRACE XPU: ', time4xi())      
#   print('=' *33)      
#   trace2xpu = None
#   try:
#       trace2xpu = trace2image4xpu() 
#       print('(Step 15B/25) OF XPU2IMAGE4XI5TEST - TRACE2XPU: ', trace2xpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4xpu', 'METHOD': 'trace2image4xpu'}
#   except:
#       print('(Step 15Z1/25) OF XPU2IMAGE4XI5TEST *** FAILED TO TRACE XPU: ', trace2xpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4xpu(task4trace2xpu=None):  

    '''
    creates callback to xpu2image4xi
    '''
    
    global xpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4xpu" 
    method2print = "trace2image4xpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4xpu-started: " + str(trace2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4xpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2xpu-to-trace2image4xpu: " + str(task4trace2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    xpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2xpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY XPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2xpu = {}                                                                                            
            
        task4trace2xpu['CLASS']   = "trace2image4xpu"                                                               # XPUIZE ZPU TRACE!
        task4trace2xpu['METHOD']  = "trace2image4xpu" 
        task4trace2xpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4xpu-failed " + str(xpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2xpu-set-by-trace2image4xpu to: " + str(task4trace2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4xpu',
        #  'METHOD': 'trace2image4xpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY XPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2xpu-try-by-trace2image4xpu: " + str(task4trace2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4xpu : trace2image4xpu ; (Step 3A/33) try; task4trace2xpu-try-by-trace2image4xpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4xpu',
    #  'METHOD': 'trace2image4xpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        xpu2trace4xi ['CLASS']  = task4trace2xpu['CLASS']                                                      # XPUIZE XPU TRACE!
        xpu2trace4xi ['METHOD']  = task4trace2xpu['METHOD']
        xpu2trace4xi ['MESSAGE'] = task4trace2xpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; xpu2trace4xi-set-by-trace2image4xpu to: " + str(xpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4xpu : trace2image4xpu ; (Step 3B/33) set; xpu2trace4xi-set-by-trace2image4xpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4xpu',
        #                      'METHOD': 'trace2image4xpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; xpu-trace-failed-by-trace2image4xpu: " + str(task4trace2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE XPU TRACE
        # =================================
        class2print   = task4trace2xpu['CLASS']                                                                # XPUIZE XPU TRACE!
        method2print  = task4trace2xpu['METHOD']
        
        message2print = task4trace2xpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # XPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4xpu : trace2image4xpu ; (Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4xpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4xpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-xpu-trace-failed-by-trace2image4xpu: " + str(task4trace2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4xpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4xpu"                
        message2print = "{(Step 3D/33) performed" + "; xpu-trace-performed-by-trace2image4xpu: " + str(task4trace2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4xpu : trace2image4xpu ; (Step 3D/33) performed; xpu-trace-performed-by-trace2image4xpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4xpu', 'METHOD': 'trace2image4xpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; xpu-trace-failed-by-trace2image4xpu: " + str(task4trace2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; xpu2trace4xi-returning-by-trace2image4xpu: " + str(xpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4xpu', 
    #  'METHOD': 'trace2image4xpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return xpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4xpu" usage = "
#
#   print('(Step 14A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF XPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4xpu()                                                                                                 
#       print('(Step 14B/700) OF XPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF XPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4xpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF XPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4xpu(task4set2imagecallback=None):

    '''
    registers callback function to xpu2image4xi
    '''
    global xpu2image4xi
    
    class2print = "set2imagecallback4xpu" 
    method2print = "set2imagecallback4xpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4xpu-started: " + str(set2imagecallback4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4xpu at 0x000000000390B748>
    
    try:
    
        xpu2image4xi ['CALLBACK'] = trace2image4xpu
        
        message2print = "{(Step 1B/70) set callback2xpu" + "; imagecallback4xpu-set-by-set2imagecallback4xpu: " + str(xpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4xpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-xpu2image4xi-returning-by-set2imagecallback4xpu: " + str(xpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4xpu at 0x0000000003864A48> 
        
        return xpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4xpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4xpu-returning-by-set2imagecallback4xpu: " + str(trace2image4xpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4xpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4xpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF XPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4XPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4xpu()                                                                 
#       print('(Step 2B/700) OF XPU2IMAGE4XI5TEST - CREATED 2D IMAGE4XPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4XPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF XPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for XPU ', time4xi()) 
#   # 
#   try:
#       xi2xpu = XPU2Image4Xi()                            
#       response4create2Dimage = xi2xpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF XPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for XPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4xpu(task4imagecreate2xpu=None):

    '''
    creates 2D image for XPU
    '''
    
    global xpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4xpu" 
    method2print = "create2image4xpu"

    message2print = "{(Step 1A/70) started" + "; create2image4xpu-started: " + str(create2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4xpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2xpu-to-create2image4xpu: " + str(task4imagecreate2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    xpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2xpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO XPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2xpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2xpu-created-by-create2image4xpu to: " + str(task4imagecreate2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4xpu = trace2image4xpu                                                                 # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4xpu-created-by-create2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4xpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4xpu"                                                          # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4xpu"                                                          # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4xpu',
            #  'METHOD': 'create2image4xpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2xpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2xpu-created-by-create2image4xpu to: " + str(task4imagecreate2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4xpu',
            #  'METHOD': 'create2image4xpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2xpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-create2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-create2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4xpu',
                #  'METHOD': 'create2context4xpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-create2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2xpu ['CONTEXT'] = context2xpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-create2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-create2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_XPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-create2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2xpu ['DEVICE'] = device2xpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2xpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2xpu-trying-by-create2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2xpu-got-by-create2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2xpu-failed-by-create2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2xpu ['QUEUE'] = queue2xpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2xpu-trying-by-create2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2xpu-got-by-create2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2xpu-failed-by-create2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2xpu ['PIPE'] = pipe2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2xpu-trying-by-create2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2xpu-got-by-create2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4xpu',
                #  'METHOD': 'create2commandqueue4xpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4xpu', 
                #  'METHOD': 'create2context4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2xpu-failed-by-create2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2xpu-trying-by-create2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                                   # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2xpu-got-by-create2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4xpu',
                #                    'METHOD': 'create2buffer4xpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4xpu', 
                #  'METHOD': 'create2context4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2xpu-failed-by-create2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2xpu ['BUFFER'] = bufferinfo2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2xpu-trying-by-create2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2xpu-got-by-create2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4xpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4xpu',
                #                    'METHOD': 'create2event4xpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4xpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4xpu', 'METHOD': 'create2context4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2xpu-failed-by-create2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2xpu ['EVENT'] = eventinfo2xpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2xpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2xpu-trying-by-create2image4xpu: " + str(flags2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2xpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2xpu-set-by-create2image4xpu to: " + str(flags2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2xpu-failed-by-create2image4xpu: " + str(flags2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2xpu ['FLAGS'] = flags2xpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2xpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2xpu-trying-by-create2image4xpu: " + str(width2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2xpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2xpu = 2 * width2xpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2xpu-set-by-create2image4xpu to: " + str(width2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2xpu-failed-by-create2image4xpu: " + str(width2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2xpu ['WIDTH'] = width2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2xpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2xpu-trying-by-create2image4xpu: " + str(height2xpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2xpu = height2layer * quantity2total
                
                height2xpu = 2 * height2xpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2xpu-set-by-create2image4xpu to: " + str(height2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2xpu-failed-by-create2image4xpu: " + str(height2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2xpu ['HEIGHT'] = height2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2xpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2xpu-trying-by-create2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2xpu-set-by-create2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2xpu-failed-by-create2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2xpu ['ROWPITCH'] = rowpitch2xpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2xpu = None
            size2xpu = int(width2xpu * height2xpu)                                                                  # CALCULATED MEMORY
            if size2xpu == 0:
                size2xpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2xpu-trying-by-create2image4xpu: " + str(pointer2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2xpu = Arena4Xi(size2xpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2xpu-got-by-create2image4xpu: " + str(arena2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2xpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4xpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2xpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4xpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2xpu:
                    size2xpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2xpu-set-by-create2buffer4xpu to: " + str(size2xpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2xpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2xpu-set-by-create2buffer4xpu to: " + str(pointer2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2xpu-failed-by-create2image4xpu: " + str(pointer2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2xpu ['POINTER'] = pointer2xpu                            
            task4imagecreate2xpu ['SIZE']    = size2xpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2xpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2xpu-trying-by-create2image4xpu: " + str(format2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2xpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2xpu-set-by-create2image4xpu to: " + str(format2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2xpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2xpu-set-by-create2image4xpu to: " + str(format2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2xpu-failed-by-create2image4xpu: " + str(format2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2xpu ['FORMAT'] = format2xpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2xpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2xpu-set-by-create2image4xpu to: " + str(depth2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2xpu ['DEPTH'] = depth2xpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2xpu-yielded-by-create2image4xpu: " + str(task4imagecreate2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2xpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR XPU! 
    # ---------------------------------------------------------------------------------------------    
    xpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; xpu2image4xi-initialized-by-create2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['DATA2CALLBACK'] = task4imagecreate2xpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4xpu to: " + str(xpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4xpu',
        #  'METHOD': 'create2image4xpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2xpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4xpu to: " + str(xpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4xpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['PLATFORM'] = task4imagecreate2xpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4xpu to: " + str(xpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2xpu-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['CONTEXT'] = task4imagecreate2xpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4xpu to: " + str(xpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4xpu',
        #  'METHOD': 'create2context4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['DEVICE'] = task4imagecreate2xpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4xpu to: " + str(xpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['CODE2ERROR'] = task4imagecreate2xpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4xpu to: " + str(xpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['QUEUE'] = task4imagecreate2xpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4xpu to: " + str(xpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['PIPE'] = task4imagecreate2xpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4xpu to: " + str(xpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-xpu2image4xi-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['COMMANDQUEUE']  = task4imagecreate2xpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4xpu to: " + str(xpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4xpu',
        #  'METHOD': 'create2commandqueue4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['BUFFER']  = task4imagecreate2xpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4xpu to: " + str(xpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4xpu',
        #  'METHOD': 'create2buffer4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['EVENT']  = task4imagecreate2xpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4xpu to: " + str(xpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4xpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4xpu',
        #                     'METHOD': 'create2event4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4xpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['FLAGS']  = task4imagecreate2xpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4xpu to: " + str(xpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        xpu2image4xi['WIDTH']  = task4imagecreate2xpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4xpu to: " + str(xpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        xpu2image4xi['HEIGHT']  = task4imagecreate2xpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4xpu to: " + str(xpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['ROWPITCH']  = task4imagecreate2xpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4xpu to: " + str(xpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4xpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['POINTER']  = task4imagecreate2xpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4xpu to: " + str(xpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['FORMAT']  = task4imagecreate2xpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4xpu to: " + str(xpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['DEPTH']  = task4imagecreate2xpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4xpu to: " + str(xpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    xpu2image4xi ['ACK2RETURN']  = code2ack
    xpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; xpu2image4xi-returning-by-create2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return xpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4xpu" usage = "
#    
#   print('(Step 4B/33) OF XPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4XPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4XPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4XPU5TEST - TRYING TO INITIALIZE DEVICE4XPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2xpu = init2xpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4XPU5TEST - INITIALIZED DEVICE4XPU TO: ', device2xpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4XPU5TEST *** FAILED TO INITIALIZE DEVICE4XPU', time4xi()) 
#   print('(Step 3A/33) OF XPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4XPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2xpu = create2context4xpu()                                                                               
#       print('(Step 3B/33) OF XPU2IMAGE4XI5TEST - CREATED CONTEXT4XPU: ', context2xpu)                                  
#   except:
#       print('(Step 3Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4XPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2xpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4XPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4xpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF XPU2IMAGE4XI5TEST - SET 2D IMAGE4XPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4XPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4xpu(task4imageset2xpu=None):

    '''
    sets 2D images for XPU
    '''
    global xpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4xpu"
    method2print = "set2image4xpu"

    message2print = "{(Step 1A/33) started" + "; set2image4xpu-started: " + str(set2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4xpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2xpu-to-set2image4xpu: " + str(task4imageset2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4xpu',
    #              'METHOD': 'create2context4xpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4xpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4xpu',
    #              'METHOD': 'create2commandqueue4xpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4xpu',
    #              'METHOD': 'create2context4xpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4xpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) set" + "; xpu2image4xi-set-by-set2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-set2image4xpu: " + str(create2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4xpu at 0x0000000003A627C8>;                                  
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-set2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['DATA2CALLBACK'] = task4imageset2xpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4xpu to: " + str(xpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2xpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4xpu to: " + str(xpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['PLATFORM'] = task4imageset2xpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4xpu to: " + str(xpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2xpu-try-by-set2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['CONTEXT'] = task4imageset2xpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4xpu to: " + str(xpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4xpu',
        #                    'METHOD': 'create2context4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['DEVICE'] = task4imageset2xpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4xpu to: " + str(xpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_XPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['CODE2ERROR'] = task4imageset2xpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4xpu to: " + str(xpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['QUEUE'] = task4imageset2xpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4xpu to: " + str(xpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['PIPE'] = task4imageset2xpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4xpu to: " + str(xpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-xpu2image4xi-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['COMMANDQUEUE']  = task4imageset2xpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4xpu to: " + str(xpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['BUFFER']  = task4imageset2xpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4xpu to: " + str(xpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4xpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['EVENT']  = task4imageset2xpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4xpu to: " + str(xpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['FLAGS']  = task4imageset2xpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4xpu to: " + str(xpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        xpu2image4xi['WIDTH']  = task4imageset2xpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4xpu to: " + str(xpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4xpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        xpu2image4xi['HEIGHT']  = task4imageset2xpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4xpu to: " + str(xpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['ROWPITCH']  = task4imageset2xpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4xpu to: " + str(xpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['POINTER']  = task4imageset2xpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4xpu to: " + str(xpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4xpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        xpu2image4xi['FORMAT']  = task4imageset2xpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4xpu to: " + str(xpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    xpu2image4xi ['ACK2RETURN']  = code2ack
    xpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; xpu2image4xi-returning-by-set2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return xpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4xpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4XPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4XPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4xpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4XPU : ', response4copy2image)
#       queue2copy4xpu      = response4copy2image['QUEUE']
#       channel2copypublish4xpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4xpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4XPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2copyimagesend4xpu = "<copy4xi>copy2copy4xpu</copy4xi>"
#       channel2copypublish4xpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4xpu.send(message2copyimagesend4xpu)
#       print("(Step 17B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copypublish4xpu, " MESSAGE2SEND4XPU: " , message2copyimagesend4xpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4XPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2copyimage4xpu(queue2copy4xpu, channel2copyconsume4xpu, channel2copypublish4xpu)
#       print("(Step 18D/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2copyimage4xpu)
#   except:
#       print("(Step 18Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())     
#   try:
#       channel2copyconsume4xpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copyconsume4xpu.recv()
#       print("(Step 19B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # <roger4xi>roger2copy4xpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4XPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2copyimagesend4xpu = "<copy4xi>copy2copy4xpu</copy4xi>"
#       channel2copypublish4xpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4xpu.send(message2copyimagesend4xpu)
#       print("(Step 20B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copypublish4xpu, " MESSAGE2SEND4XPU: " , message2copyimagesend4xpu)                                                                                                
#       # <copy4xi>copy2copy4xpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4XPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2copyimage4xpu(queue2copy4xpu, channel2copyconsume4xpu, channel2copypublish4xpu)
#       print("(Step 21B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2copyimage4xpu)
#   except:
#       print("(Step 21Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4XPU ', time4xi())     
#   try:
#       channel2copyconsume4xpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copyconsume4xpu.recv()
#       print("(Step 22B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # <copy4xi>copy2copy4xpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4XPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4XPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2copyimage4xpu(queue2copy4xpu, channel2copyconsume4xpu, channel2copypublish4xpu)
#       print("(Step 25B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2copyimage4xpu)
#   except:
#       print("(Step 25Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4xpu = channel2copyconsume4xpu.recv()
#       print("(Step 26D/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4xpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4XPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2copyimage4xpu(queue2copy4xpu, channel2copyconsume4xpu, channel2copypublish4xpu)
#       print("(Step 27B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvre4xpu)
#   except:
#       print("(Step 27Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4xpu(task4imagecopy2xpu=None):

    '''
    copies 2D images for XPU
    '''
    global xpu2image4xi                                                                                             # 2D-IMAGES FOR XPU  
    global control2copyimage4xpu                                                                                    # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4xpu"
    method2print = "copy2image4xpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4xpu-started: " + str(copy2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4xpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2xpu-to-copy2image4xpu: " + str(task4imagecopy2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                       # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 0C/70) created" + "; xpu2image4xi-created-by-copy2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4xpu-skipped-by-copy2image4xpu: " + str(copy2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4xpu at 0x00000000038538C8>                                 
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 0E/70) set" + "; xpu2image4xi-set-by-copy2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; xpu2image4xi-yielded-by-copy2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2xpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2xpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2xpu-created-by-copy2image4xpu to: " + str(task4imagecopy2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                                 # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-copy2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4xpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4xpu"                                                            # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4xpu"                                                            # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4xpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4xpu',
            #  'METHOD': 'copy2image4xpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2xpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2xpu-created-by-copy2image4xpu to: " + str(task4imagecopy2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4xpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4xpu',
            #  'METHOD': 'copy2image4xpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2xpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-copy2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-copy2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-copy2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2xpu ['CONTEXT'] = context2xpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-copy2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-copy2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-copy2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2xpu ['DEVICE'] = device2xpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2xpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2xpu-trying-by-copy2image4xpu: " + str(imagesourceinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2xpu-got-by-copy2image4xpu: " + str(imagesourceinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2xpu-failed-by-copy2image4xpu: " + str(imagesourceinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2xpu ['IMAGESOURCE'] = imagesourceinfo2xpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2xpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2xpu-trying-by-copy2image4xpu: " + str(imagedestinationinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2xpu-got-by-copy2image4xpu: " + str(imagedestinationinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2xpu-failed-by-copy2image4xpu: " + str(imagedestinationinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2xpu ['IMAGEDESTINATION'] = imagedestinationinfo2xpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2xpu-trying-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2xpu-got-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2xpu-failed-by-copy2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2xpu ['QUEUE'] = queue2xpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2xpu-trying-by-copy2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2xpu-got-by-copy2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2xpu-failed-by-copy2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2xpu ['PIPE'] = pipe2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2xpu-got-by-copy2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4xpu', 
                #  'METHOD': 'create2commandqueue4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2xpu-trying-by-copy2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                               # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2xpu-got-by-copy2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2xpu-failed-by-copy2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2xpu ['BUFFER'] = bufferinfo2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2xpu-trying-by-copy2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2xpu-got-by-copy2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4xpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4xpu',
                #  'METHOD': 'create2event4xpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4xpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2xpu-failed-by-copy2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2xpu ['EVENT'] = eventinfo2xpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2xpu-set-by-copy2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2xpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2xpu-trying-by-copy2image4xpu: " + str(origin4source2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2xpu['X'] = 0 
                origin4source2xpu['Y'] = 0  
                origin4source2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2xpu-set-by-copy2image4xpu to: " + str(origin4source2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2xpu-failed-by-copy2image4xpu: " + str(origin4source2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2xpu ['SOURCEORIGIN'] = origin4source2xpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2xpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2xpu-trying-by-copy2image4xpu: " + str(origin4destination2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2xpu['X'] = 100 
                origin4destination2xpu['Y'] = 100  
                origin4destination2xpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2xpu-set-by-copy2image4xpu to: " + str(origin4destination2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2xpu-failed-by-copy2image4xpu: " + str(origin4destination2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2xpu ['DESTINATIONORIGIN'] = origin4destination2xpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2xpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2xpu-trying-by-copy2image4xpu: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2xpu['X'] = 10 
                region4copy2xpu['Y'] = 10  
                region4copy2xpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2xpu-set-by-copy2image4xpu to: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2xpu-failed-by-copy2image4xpu: " + str(region4copy2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2xpu ['REGION'] = region4copy2xpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2xpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                            # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-copy2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4xpu"                                                        # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4xpu"                                                        # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2xpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2xpu-created-by-copy2image4xpu to: " + str(task4imagecopy2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2xpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-copy2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-copy2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-copy2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2xpu ['CONTEXT'] = context2xpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-copy2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-copy2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-copy2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2xpu ['DEVICE'] = device2xpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2xpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2xpu-trying-by-copy2image4xpu: " + str(sourceimageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2xpu-got-by-copy2image4xpu: " + str(sourceimageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2xpu-failed-by-copy2image4xpu: " + str(sourceimageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2xpu ['IMAGESOURCE'] = sourceimageinfo2xpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2xpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2xpu-trying-by-copy2image4xpu: " + str(destinationimageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2xpu-got-by-copy2image4xpu: " + str(destinationimageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2xpu-failed-by-copy2image4xpu: " + str(destinationimageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2xpu ['IMAGEDESTINATION'] = destinationimageinfo2xpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2xpu-trying-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2xpu-got-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2xpu-failed-by-copy2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2xpu ['QUEUE'] = queue2xpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2xpu-trying-by-copy2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2xpu-got-by-copy2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2xpu-failed-by-copy2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2xpu ['PIPE'] = pipe2xpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2xpu-got-by-copy2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2xpu-trying-by-copy2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                           # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2xpu-got-by-copy2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2xpu-failed-by-copy2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2xpu ['BUFFER'] = bufferinfo2xpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2xpu-trying-by-copy2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2xpu-got-by-copy2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2xpu-failed-by-copy2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2xpu ['EVENT'] = eventinfo2xpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2xpu-set-by-copy2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2xpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2xpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2xpu-trying-by-copy2image4xpu: " + str(source4origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2xpu['X'] = 0 
                source4origin2xpu['Y'] = 0  
                source4origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2xpu-set-by-copy2image4xpu to: " + str(source4origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2xpu-failed-by-copy2image4xpu: " + str(source4origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2xpu ['SOURCEORIGIN'] = source4origin2xpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2xpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2xpu-trying-by-copy2image4xpu: " + str(destination4origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2xpu['X'] = 100 
                destination4origin2xpu['Y'] = 100  
                destination4origin2xpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2xpu-set-by-copy2image4xpu to: " + str(destination4origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2xpu-failed-by-copy2image4xpu: " + str(destination4origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2xpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2xpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2xpu-trying-by-copy2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2xpu-set-by-copy2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2xpu-failed-by-copy2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2xpu ['REGION'] = region2xpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2xpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2xpu-yielded-by-copy2image4xpu: " + str(task4imagecopy2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-copy2image4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2xpu, publisherchannel2xpu = task4imagecopy2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2xpu-got-by-copy2image4xpu: " + str(consumerchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2xpu-got-by-copy2image4xpu: " + str(publisherchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2xpu = True
        
        control2copyimage4xpu['CONSUMERCHANNEL']  = consumerchannel2xpu
        control2copyimage4xpu['PUBLISHERCHANNEL'] = publisherchannel2xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2xpu = False
    queue2xpu = None
    message2print = "{(Step 41A/70) trying" + "queue2xpu-trying-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2xpu = task4imagecopy2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2xpu-got-by-copy2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2xpu = True
        control2copyimage4xpu['QUEUE']  = queue2xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-copy2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2copyimage4xpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2xpu, publisherchannel2xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-copy2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4xpu-returning-by-copy2image4xpu: " + str(control2copyimage4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4xpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4xpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4XPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4XPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4xpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4XPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4xpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4xpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4xpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4XPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4xpu = "<copy4xi>copy2copyimagetobuffer4xpu</copy4xi>"
#       channel2copyimagetobufferpublish4xpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4xpu.send(message2copyimagetobuffersend4xpu)
#       print("(Step 61B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copyimagetobufferpublish4xpu, " MESSAGE2SEND4XPU: " , message2copyimagetobuffersend4xpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copyimagetobuffer4xpu(queue2copyimagetobuffer4xpu, channel2copyimagetobufferconsume4xpu, channel2copyimagetobufferpublish4xpu)
#       print("(Step 62B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copyimagetobuffer4xpu)
#   except:
#       print("(Step 62Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4xpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copyimagetobufferconsume4xpu.recv()
#       print("(Step 63B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyimagetobufferconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4XPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4xpu = "<copy4xi>copy2copyimagetobuffer4xpu</copy4xi>"
#       channel2copyimagetobufferpublish4xpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4xpu.send(message2copyimagetobuffersend4xpu)
#       print("(Step 64B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copyimagetobufferpublish4xpu, " MESSAGE2SEND4XPU: " , message2copyimagetobuffersend4xpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copyimagetobuffer4xpu(queue2copyimagetobuffer4xpu, channel2copyimagetobufferconsume4xpu, channel2copyimagetobufferpublish4xpu)
#       print("(Step 65B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copyimagetobuffer4xpu)
#   except:
#       print("(Step 65Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4XPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4xpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copyimagetobufferconsume4xpu.recv()
#       print("(Step 66B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyimagetobufferconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4XPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copyimagetobuffer4xpu(queue2copyimagetobuffer4xpu, channel2copyimagetobufferconsume4xpu, channel2copyimagetobufferpublish4xpu)
#       print("(Step 68B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copyimagetobuffer4xpu)
#   except:
#       print("(Step 68Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4xpu = channel2copyimagetobufferconsume4xpu.recv()
#       print("(Step 69D/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copyimagetobufferconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copyimagetobuffer4xpu(queue2copyimagetobuffer4xpu, channel2copyimagetobufferconsume4xpu, channel2copyimagetobufferpublish4xpu)
#       print("(Step 70B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvreimagecopytobuffer4xpu)
#   except:
#       print("(Step 70Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4xpu(task4copyimagetobuffer2xpu=None):

    '''
    copies 2D images to buffer for XPU
    '''
    global xpu2image4xi                                                                                             # 2D-IMAGES FOR XPU  
    global xpu2buffer4xi                                                                                            # BUFFER FOR XPU
    
    global control2copyimagetobuffer4xpu                                                                            # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4xpu"
    method2print = "copy2imagetobuffer4xpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4xpu-started: " + str(copy2imagetobuffer4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4xpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2xpu-to-copy2imagetobuffer4xpu: " + str(task4copyimagetobuffer2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) created" + "; xpu2image4xi-created-by-copy2imagetobuffer4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-copy2imagetobuffer4xpu: " + str(copy2imagetobuffer4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4xpu-skipped-by-copy2imagetobuffer4xpu                                 
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-copy2imagetobuffer4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; xpu2image4xi-yielded-by-copy2imagetobuffer4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2buffer4xi is None:
            xpu2buffer4xi = create2buffer4xpu()                                                                     # INITIALIZE BUFFER FOR XPU 
            message2print = "{(Step 1F/70) created" + "; xpu2buffer4xi-created-by-copy2imagetobuffer4xpu to: " + str(xpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4xpu-skipped-by-copy2imagetobuffer4xpu: " + str(copy2imagetobuffer4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            xpu2buffer4xi = create2buffer4xpu()
            message2print = "{(Step 1H/70) set" + "; xpu2buffer4xi-set-by-copy2imagetobuffer4xpu to: " + str(xpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4xpu',
            #   'METHOD': 'create2buffer4xpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            xpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; xpu2buffer4xi-yielded-by-copy2imagetobuffer4xpu: " + str(xpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2xpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2xpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2xpu-created-by-copy2imagetobuffer4xpu to: " + str(task4copyimagetobuffer2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4xpu = trace2image4xpu                                                                 # CALLBACK FUNCTION FOR XPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-copy2imagetobuffer4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4xpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4xpu"                                                            # MODULE OF XPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4xpu"                                                            # FUNCTION OF XPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4xpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4xpu', 
        #  'METHOD': 'copy2imagetobuffer4xpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2xpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2xpu-created-by-copy2imagetobuffer4xpu to: " + str(task4copyimagetobuffer2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4xpu', 
        #  'METHOD': 'copy2imagetobuffer4xpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2xpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2xpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-copy2imagetobuffer4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-copy2imagetobuffer4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4xpu', 
            #  'METHOD': 'create2context4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-copy2imagetobuffer4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2xpu ['CONTEXT'] = context2xpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2xpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-copy2imagetobuffer4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-copy2imagetobuffer4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_XPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-copy2imagetobuffer4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2xpu ['DEVICE'] = device2xpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2xpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(imagesourceinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2xpu = get2imageinfo4xpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(imagesourceinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(imagesourceinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2xpu ['IMAGESOURCE'] = imagesourceinfo2xpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2xpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(bufferdestinationinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2xpu = get2bufferinfo4xpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(bufferdestinationinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4xpu', 
            #  'METHOD': 'create2buffer4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(bufferdestinationinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2xpu ['BUFFERDESTINATION'] = bufferdestinationinfo2xpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY XPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4xpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4xpu-trying-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4xpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4xpu-got-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4xpu-failed-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2xpu ['QUEUE'] = queue2copyimagetobuffer4xpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY XPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2xpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2xpu-trying-by-copy2imagetobuffer4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2xpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2xpu-got-by-copy2imagetobuffer4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2xpu-failed-by-copy2imagetobuffer4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2xpu ['PIPE'] = pipe2xpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2xpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4xpu', 
            #  'METHOD': 'create2commandqueue4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2xpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2xpu = get2eventinfo4xpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4xpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4xpu', 
            #  'METHOD': 'create2event4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4xpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2xpu ['EVENT'] = eventinfo2xpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2xpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2xpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2xpu-set-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2xpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2xpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2xpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2xpu-trying-by-copy2imagetobuffer4xpu: " + str(origin4source2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2xpu['X'] = 0 
            origin4source2xpu['Y'] = 0  
            origin4source2xpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2xpu-set-by-copy2imagetobuffer4xpu to: " + str(origin4source2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2xpu-failed-by-copy2imagetobuffer4xpu: " + str(origin4source2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2xpu ['SOURCEORIGIN'] = origin4source2xpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2xpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2xpu-trying-by-copy2imagetobuffer4xpu: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2xpu['X'] = 10 
            region4copy2xpu['Y'] = 10  
            region4copy2xpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2xpu-set-by-copy2imagetobuffer4xpu to: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2xpu-failed-by-copy2imagetobuffer4xpu: " + str(region4copy2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2xpu ['REGION'] = region4copy2xpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2xpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2xpu-set-by-copy2imagetobuffer4xpu to: " + str(offset4copy2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2xpu ['OFFSET'] = offset4copy2xpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2xpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4xpu = trace2image4xpu                                                             # CALLBACK FUNCTION FOR XPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-copy2imagetobuffer4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4xpu"                                                        # MODULE OF XPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4xpu"                                                        # FUNCTION OF XPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2xpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2xpu-created-by-copy2imagetobuffer4xpu to: " + str(task4copyimagetobuffer2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2xpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2xpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-copy2imagetobuffer4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2xpu = get2context4xpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-copy2imagetobuffer4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-copy2imagetobuffer4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2xpu ['CONTEXT'] = context2xpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2xpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-copy2imagetobuffer4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2xpu = get2xpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-copy2imagetobuffer4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-copy2imagetobuffer4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2xpu ['DEVICE'] = device2xpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2xpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(sourceimageinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2xpu = get2imageinfo4xpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(sourceimageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(sourceimageinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2xpu ['IMAGESOURCE'] = sourceimageinfo2xpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2xpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(destinationbufferinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2xpu = get2bufferinfo4xpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(destinationbufferinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(destinationbufferinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2xpu ['BUFFERDESTINATION'] = destinationbufferinfo2xpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY XPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4xpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4xpu-trying-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4xpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4xpu-got-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4xpu-failed-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2xpu ['QUEUE'] = queue2copyimagetobuffer4xpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY XPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2xpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2xpu-trying-by-copy2imagetobuffer4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2xpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2xpu-got-by-copy2imagetobuffer4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2xpu-failed-by-copy2imagetobuffer4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2xpu ['PIPE'] = pipe2xpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2xpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2xpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2xpu = get2eventinfo4xpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2xpu-got-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2xpu ['EVENT'] = eventinfo2xpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2xpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2xpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2xpu-set-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2xpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2xpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2imagetobuffer4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2xpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2xpu-trying-by-copy2imagetobuffer4xpu: " + str(source4origin2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2xpu['X'] = 0 
            source4origin2xpu['Y'] = 0  
            source4origin2xpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2xpu-set-by-copy2imagetobuffer4xpu to: " + str(source4origin2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2xpu-failed-by-copy2imagetobuffer4xpu: " + str(source4origin2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2xpu ['SOURCEORIGIN'] = source4origin2xpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2xpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2xpu-trying-by-copy2imagetobuffer4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2xpu-set-by-copy2imagetobuffer4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2xpu-failed-by-copy2imagetobuffer4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2xpu ['REGION'] = region2xpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2xpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2xpu-set-by-copy2imagetobuffer4xpu to: " + str(offset2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2xpu ['OFFSET'] = offset2xpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2xpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-copy2imagetobuffer4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4xpu, publisherchannel2copyimagetobuffer4xpu = task4copyimagetobuffer2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4xpu-got-by-copy2imagetobuffer4xpu: " + str(consumerchannel2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4xpu-got-by-copy2imagetobuffer4xpu: " + str(publisherchannel2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2xpu = True
        
        control2copyimagetobuffer4xpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4xpu
        
        
        control2copyimagetobuffer4xpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4xpu = False
    queue2copyimagetobuffer4xpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4xpu-trying-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4xpu = task4copyimagetobuffer2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4xpu-got-by-copy2imagetobuffer4xpu: " + str(queue2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4xpu = True
        control2copyimagetobuffer4xpu['QUEUE']  = queue2copyimagetobuffer4xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-copy2imagetobuffer4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4xpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4xpu, publisherchannel2copyimagetobuffer4xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-copy2imagetobuffer4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4xpu-returning-by-copy2imagetobuffer4xpu: " + str(control2copyimagetobuffer4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4xpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4xpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4XPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4XPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4xpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4XPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4xpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4xpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4xpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4XPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4xpu = "<copy4xi>copy2copybuffertoimage4xpu</copy4xi>"
#       channel2copybuffertoimagepublish4xpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4xpu.send(message2copybuffertoimagesend4xpu)
#       print("(Step 81B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copybuffertoimagepublish4xpu, " MESSAGE2SEND4XPU: " , message2copybuffertoimagesend4xpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copybuffertoimage4xpu(queue2copybuffertoimage4xpu, channel2copybuffertoimageconsume4xpu, channel2copybuffertoimagepublish4xpu)
#       print("(Step 82D/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copybuffertoimage4xpu)
#   except:
#       print("(Step 82Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4xpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copybuffertoimageconsume4xpu.recv()
#       print("(Step 83B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copybuffertoimageconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4XPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4xpu = "<copy4xi>copy2copybuffertoimage4xpu</copy4xi>"
#       channel2copybuffertoimagepublish4xpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4xpu.send(message2copybuffertoimagesend4xpu)
#       print("(Step 84B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2copybuffertoimagepublish4xpu, " MESSAGE2SEND4XPU: " , message2copybuffertoimagesend4xpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copybuffertoimage4xpu(queue2copybuffertoimage4xpu, channel2copybuffertoimageconsume4xpu, channel2copybuffertoimagepublish4xpu)
#       print("(Step 85B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copybuffertoimage4xpu)
#   except:
#       print("(Step 85Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4XPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4xpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4xpu = channel2copybuffertoimageconsume4xpu.recv()
#       print("(Step 86B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copybuffertoimageconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4XPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copybuffertoimage4xpu(queue2copybuffertoimage4xpu, channel2copybuffertoimageconsume4xpu, channel2copybuffertoimagepublish4xpu)
#       print("(Step 88B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", manoeuvre2copybuffertoimage4xpu)
#   except:
#       print("(Step 88Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4xpu = channel2copybuffertoimageconsume4xpu.recv()
#       print("(Step 89D/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2copybuffertoimageconsume4xpu, " CONTROL & COMMAND: ", message2copyimagereceive4xpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4XPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4xpu = manoeuvre2copybuffertoimage4xpu(queue2copybuffertoimage4xpu, channel2copybuffertoimageconsume4xpu, channel2copybuffertoimagepublish4xpu)
#       print("(Step 90B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvreimagecopytobuffer4xpu)
#   except:
#       print("(Step 90Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4XPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4xpu(task4copybuffertoimage2xpu=None):

    '''
    copies buffer to image for XPU
    '''
    global xpu2image4xi                                                                                             # 2D-IMAGES FOR XPU  
    global xpu2buffer4xi                                                                                            # BUFFER FOR XPU
    
    global control2copybuffertoimage4xpu                                                                            # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4xpu"
    method2print = "copy2buffertoimage4xpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4xpu-started: " + str(copy2buffertoimage4xpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4xpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2xpu-to-copy2buffertoimage4xpu: " + str(task4copybuffertoimage2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) created" + "; xpu2image4xi-created-by-copy2buffertoimage4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-copy2buffertoimage4xpu: " + str(copy2buffertoimage4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-copy2buffertoimage4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; xpu2image4xi-yielded-by-copy2buffertoimage4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2buffer4xi is None:
            xpu2buffer4xi = create2buffer4xpu()                                                                     # INITIALIZE BUFFER FOR XPU 
            message2print = "{(Step 1F/70) created" + "; xpu2buffer4xi-created-by-copy2buffertoimage4xpu to: " + str(xpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4xpu-skipped-by-copy2buffertoimage4xpu: " + str(copy2buffertoimage4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            xpu2buffer4xi = create2buffer4xpu()
            message2print = "{(Step 1H/70) set" + "; xpu2buffer4xi-set-by-copy2buffertoimage4xpu to: " + str(xpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; xpu2buffer4xi-yielded-by-copy2buffertoimage4xpu: " + str(xpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2xpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2xpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2xpu-created-by-copy2buffertoimage4xpu to: " + str(task4copybuffertoimage2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4xpu = trace2image4xpu                                                                 # CALLBACK FUNCTION FOR XPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-copy2buffertoimage4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4xpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4xpu"                                                     # MODULE OF XPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4xpu"                                                     # FUNCTION OF XPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4xpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4xpu', 
        #  'METHOD': 'copy2buffertoimage4xpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2xpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2xpu-created-by-copy2buffertoimage4xpu to: " + str(task4copybuffertoimage2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4xpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4xpu', 
        #  'METHOD': 'copy2buffertoimage4xpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2xpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2xpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-copy2buffertoimage4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-copy2buffertoimage4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-copy2buffertoimage4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2xpu ['CONTEXT'] = context2xpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2xpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-copy2buffertoimage4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-copy2buffertoimage4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-copy2buffertoimage4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2xpu ['DEVICE'] = device2xpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2xpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(imagedestinationinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2xpu = get2imageinfo4xpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(imagedestinationinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4xpu', 
            #  'METHOD': 'create2context4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4xpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(imagedestinationinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2xpu ['IMAGEDESTINATION'] = imagedestinationinfo2xpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2xpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(buffersourceinfo2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2xpu = get2bufferinfo4xpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(buffersourceinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4xpu', 
            #  'METHOD': 'create2buffer4xpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(buffersourceinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2xpu ['BUFFERSOURCE'] = buffersourceinfo2xpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY XPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4xpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4xpu-trying-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4xpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4xpu-got-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4xpu-failed-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2xpu ['QUEUE'] = queue2copybuffertoimage4xpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY XPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2xpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2xpu-trying-by-copy2buffertoimage4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2xpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2xpu-got-by-copy2buffertoimage4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2xpu-failed-by-copy2buffertoimage4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2xpu ['PIPE'] = pipe2xpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2xpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2xpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2xpu = get2eventinfo4xpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2xpu ['EVENT'] = eventinfo2xpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2xpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2xpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2xpu-set-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2xpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2xpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2xpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2xpu-trying-by-copy2buffertoimage4xpu: " + str(origin4destination2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2xpu['X'] = 100 
            origin4destination2xpu['Y'] = 100  
            origin4destination2xpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2xpu-set-by-copy2buffertoimage4xpu to: " + str(origin4destination2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2xpu-failed-by-copy2buffertoimage4xpu: " + str(origin4destination2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2xpu ['DESTINATIONORIGIN'] = origin4destination2xpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2xpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2xpu-trying-by-copy2buffertoimage4xpu: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2xpu['X'] = 10 
            region4copy2xpu['Y'] = 10  
            region4copy2xpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2xpu-set-by-copy2buffertoimage4xpu to: " + str(region4copy2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2xpu-failed-by-copy2buffertoimage4xpu: " + str(region4copy2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2xpu ['REGION'] = region4copy2xpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2xpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2xpu-set-by-copy2buffertoimage4xpu to: " + str(offset4copy2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2xpu ['OFFSET'] = offset4copy2xpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2xpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                             # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-copy2buffertoimage4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4xpu"                                                        # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4xpu"                                                        # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2xpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2xpu-created-by-copy2buffertoimage4xpu to: " + str(task4copybuffertoimage2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2xpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-copy2buffertoimage4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-copy2buffertoimage4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-copy2buffertoimage4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2xpu ['CONTEXT'] = context2xpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-copy2buffertoimage4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-copy2buffertoimage4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-copy2buffertoimage4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2xpu ['DEVICE'] = device2xpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2xpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(destinationimageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(destinationimageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(destinationimageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2xpu ['IMAGEDESTINATION'] = destinationimageinfo2xpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2xpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(sourcebufferinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2xpu = get2bufferinfo4xpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(sourcebufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(sourcebufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2xpu ['BUFFERSOURCE'] = sourcebufferinfo2xpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4xpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4xpu-trying-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4xpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4xpu-got-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4xpu-failed-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2xpu ['QUEUE'] = queue2copybuffertoimage4xpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2xpu-trying-by-copy2buffertoimage4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2xpu-got-by-copy2buffertoimage4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2xpu-failed-by-copy2buffertoimage4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2xpu ['PIPE'] = pipe2xpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2xpu-got-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2xpu ['EVENT'] = eventinfo2xpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2xpu-set-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2xpu-set-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-copy2buffertoimage4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2xpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2xpu-trying-by-copy2buffertoimage4xpu: " + str(destination4origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2xpu['X'] = 0 
                destination4origin2xpu['Y'] = 0  
                destination4origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2xpu-set-by-copy2buffertoimage4xpu to: " + str(destination4origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2xpu-failed-by-copy2buffertoimage4xpu: " + str(destination4origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2xpu ['DESTINATIONORIGIN'] = destination4origin2xpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2xpu-trying-by-copy2buffertoimage4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2xpu-set-by-copy2buffertoimage4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2xpu-failed-by-copy2buffertoimage4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2xpu ['REGION'] = region2xpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2xpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2xpu-set-by-copy2buffertoimage4xpu to: " + str(offset2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2xpu ['OFFSET'] = offset2xpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2xpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-copy2buffertoimage4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4xpu, publisherchannel2copybuffertoimage4xpu = task4copybuffertoimage2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4xpu-got-by-copy2buffertoimage4xpu: " + str(consumerchannel2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4xpu-got-by-copy2buffertoimage4xpu: " + str(publisherchannel2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2xpu = True
        
        control2copybuffertoimage4xpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4xpu
        
        
        control2copybuffertoimage4xpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4xpu = False
    queue2copybuffertoimage4xpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4xpu-trying-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4xpu = task4copybuffertoimage2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4xpu-got-by-copy2buffertoimage4xpu: " + str(queue2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4xpu = True
        control2copybuffertoimage4xpu['QUEUE']  = queue2copybuffertoimage4xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-copy2buffertoimage4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4xpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4xpu, publisherchannel2copybuffertoimage4xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-copy2buffertoimage4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4xpu-returning-by-copy2buffertoimage4xpu: " + str(control2copybuffertoimage4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4xpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4xpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4xpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4XPU : ', response4read2image)
#       queue2read4xpu      = response4read2image['QUEUE']
#       channel2readpublish4xpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4xpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2readsend4xpu = "<copy4xi>copy2read4xpu</copy4xi>"
#       channel2readpublish4xpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4xpu.send(message2readsend4xpu)
#       print("(Step 17B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2readpublish4xpu, " MESSAGE2SEND4XPU: " , message2readsend4xpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4XPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2read4xpu, channel2readconsume4xpu, channel2readpublish4xpu)
#       print("(Step 18D/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 18Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:
#       channel2readconsume4xpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4xpu = channel2readconsume4xpu.recv()
#       print("(Step 19B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2readconsume4xpu, " CONTROL & COMMAND: ", message2readreceive4xpu)                                                                                                  
#       # <roger4xi>roger2read4xpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4XPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2readsend4xpu = "<ready4xi>ready2read4xpu</ready4xi>"
#       channel2readpublish4xpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4xpu.send(message2readsend4xpu)
#       print("(Step 20B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2readpublish4xpu, " MESSAGE2SEND4XPU: " , message2readsend4xpu)                                                                                                
#       # <ready4xi>ready2read4xpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4XPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2read4xpu, channel2readconsume4xpu, channel2readpublish4xpu)
#       print("(Step 21B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 21Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:
#       channel2readconsume4xpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4xpu = channel2readconsume4xpu.recv()
#       print("(Step 22B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2readconsume4xpu, " CONTROL & COMMAND: ", message2readreceive4xpu)                                                                                                  
#       # <ready4xi>ready2read4xpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4XPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4XPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2read4xpu, channel2readconsume4xpu, channel2readpublish4xpu)
#       print("(Step 25B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 25Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4xpu = channel2readconsume4xpu.recv()
#       print("(Step 26D/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2readconsume4xpu, " CONTROL & COMMAND: ", message2readreceive4xpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4xpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4XPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2read4xpu, channel2readconsume4xpu, channel2readpublish4xpu)
#       print("(Step 27B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvre4xpu)
#   except:
#       print("(Step 27Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4xpu(task4imageread2xpu=None):

    '''
    reads 2D images for XPU
    '''
    global xpu2image4xi                                                     # 2D-IMAGES FOR XPU  
    global control2readimage4xpu                                              # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4xpu"
    method2print = "read2image4xpu"

    message2print = "{(Step 1A/33) started" + "; read2image4xpu-started: " + str(read2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4xpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2xpu-to-read2image4xpu: " + str(task4imageread2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) created" + "; xpu2image4xi-created-by-read2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-read2image4xpu: " + str(read2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-read2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; xpu2image4xi-yielded-by-read2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2xpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2xpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2xpu-created-by-read2image4xpu to: " + str(task4imageread2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                              # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-read2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4xpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4xpu"                                                          # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4xpu"                                                          # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4xpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4xpu', 
            #  'METHOD': 'read2image4xpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2xpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2xpu-created-by-read2image4xpu to: " + str(task4imageread2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4xpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4xpu', 
            #  'METHOD': 'read2image4xpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2xpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-read2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-read2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-read2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2xpu ['CONTEXT'] = context2xpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-read2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-read2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-read2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2xpu ['DEVICE'] = device2xpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2xpu-trying-by-read2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2xpu-got-by-read2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2xpu-failed-by-read2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2xpu ['IMAGE'] = imageinfo2xpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2xpu-trying-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2xpu-got-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2xpu-failed-by-read2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2xpu ['QUEUE'] = queue2xpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2xpu-trying-by-read2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2xpu-got-by-read2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2xpu-failed-by-read2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2xpu ['PIPE'] = pipe2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2xpu-trying-by-read2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2xpu-got-by-read2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2xpu-failed-by-read2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2xpu-trying-by-read2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                                   # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2xpu-got-by-read2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2xpu-failed-by-read2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2xpu ['BUFFER'] = bufferinfo2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2xpu-trying-by-read2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2xpu-got-by-read2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2xpu-failed-by-read2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2xpu ['EVENT'] = eventinfo2xpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2xpu-trying-by-read2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2xpu-set-by-read2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2xpu-failed-by-read2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-read2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2xpu-set-by-read2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-read2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2xpu-trying-by-read2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2xpu-set-by-read2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2xpu-failed-by-read2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2xpu ['BLOCKING'] = blocking2xpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2xpu-trying-by-read2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2xpu-set-by-read2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2xpu-failed-by-read2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2xpu ['ORIGIN'] = origin2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2xpu-trying-by-read2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2xpu-set-by-read2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2xpu-failed-by-read2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2xpu ['REGION'] = region2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2xpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2xpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2xpu-trying-by-read2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2xpu-set-by-read2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2xpu-failed-by-read2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2xpu ['ROWPITCH'] = rowpitch2xpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2xpu = None
            height2xpu = 800
            size2xpu = int(width2xpu * height2xpu)                                                                  # CALCULATED MEMORY
            if size2xpu == 0:
                size2xpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2xpu-trying-by-read2image4xpu: " + str(pointer2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2xpu = Arena4Xi(size2xpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2xpu-got-by-read2image4xpu: " + str(arena2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2xpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4xpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2xpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4xpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2xpu:
                    size2xpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2xpu-set-by-read2buffer4xpu to: " + str(size2xpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2xpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2xpu-set-by-read2buffer4xpu to: " + str(pointer2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2xpu-failed-by-read2image4xpu: " + str(pointer2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2xpu ['POINTER'] = pointer2xpu                            
            task4imageread2xpu ['SIZE']    = size2xpu                                                               # AUTOMATIC BY XPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2xpu-trying-by-read2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2xpu-set-by-read2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2xpu-failed-by-read2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2xpu ['SLICEPITCH'] = slicepitch2xpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2xpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                              # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-read2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4xpu"                                                          # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4xpu"                                                          # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2xpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2xpu-created-by-read2image4xpu to: " + str(task4imageread2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2xpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-read2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-read2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-read2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2xpu ['CONTEXT'] = context2xpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-read2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-read2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-read2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2xpu ['DEVICE'] = device2xpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2xpu-trying-by-read2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2xpu-got-by-read2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2xpu-failed-by-read2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2xpu ['IMAGE'] = imageinfo2xpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2xpu-trying-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2xpu-got-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2xpu-failed-by-read2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2xpu ['QUEUE'] = queue2xpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2xpu-trying-by-read2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2xpu-got-by-read2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2xpu-failed-by-read2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2xpu ['PIPE'] = pipe2xpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2xpu-trying-by-read2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2xpu-got-by-read2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2xpu-failed-by-read2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2xpu-trying-by-read2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                                   # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2xpu-got-by-read2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2xpu-failed-by-read2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2xpu ['BUFFER'] = bufferinfo2xpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2xpu-trying-by-read2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2xpu-got-by-read2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2xpu-failed-by-read2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2xpu ['EVENT'] = eventinfo2xpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2xpu-trying-by-read2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2xpu-set-by-read2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2xpu-failed-by-read2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-read2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2xpu-set-by-read2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-read2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2xpu-trying-by-read2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2xpu-set-by-read2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2xpu-failed-by-read2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2xpu ['BLOCKING'] = blocking2xpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2xpu-trying-by-read2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2xpu-set-by-read2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2xpu-failed-by-read2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2xpu ['ORIGIN'] = origin2xpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2xpu-trying-by-read2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2xpu-set-by-read2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2xpu-failed-by-read2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2xpu ['REGION'] = region2xpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2xpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2xpu-trying-by-read2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2xpu-set-by-read2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2xpu-failed-by-read2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2xpu ['ROWPITCH'] = rowpitch2xpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2xpu = None
            size2xpu = int(width2xpu * height2xpu)                                                                  # CALCULATED MEMORY
            if size2xpu == 0:
                size2xpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2xpu-trying-by-read2image4xpu: " + str(pointer2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2xpu = Arena4Xi(size2xpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2xpu-got-by-read2image4xpu: " + str(arena2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2xpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4xpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2xpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4xpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2xpu:
                    size2xpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2xpu-set-by-read2buffer4xpu to: " + str(size2xpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2xpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2xpu-set-by-read2buffer4xpu to: " + str(pointer2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2xpu-failed-by-read2image4xpu: " + str(pointer2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2xpu ['POINTER'] = pointer2xpu                            
            task4imageread2xpu ['SIZE']    = size2xpu                                                               # AUTOMATIC BY XPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2xpu-trying-by-read2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2xpu-set-by-read2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2xpu-failed-by-read2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2xpu ['SLICEPITCH'] = slicepitch2xpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2xpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2xpu-yielded-by-read2image4xpu: " + str(task4imageread2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-read2image4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2xpu, publisherchannel2xpu = task4imageread2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2xpu-got-by-read2image4xpu: " + str(consumerchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2xpu-got-by-read2image4xpu: " + str(publisherchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2xpu = True
        
        control2readimage4xpu['CONSUMERCHANNEL']  = consumerchannel2xpu
        
        
        control2readimage4xpu['PUBLISHERCHANNEL'] = publisherchannel2xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2xpu = False
    queue2xpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2xpu-trying-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2xpu = task4imageread2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2xpu-got-by-read2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2xpu = True
        control2readimage4xpu['QUEUE']  = queue2xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-read2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2readimage4xpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2xpu, publisherchannel2xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-read2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4xpu-returning-by-read2image4xpu: " + str(control2readimage4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4xpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4xpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4XPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4xpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4XPU : ', response4map2image)
#    
#       queue4map2image4xpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4xpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4xpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4XPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4XPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())      
#   try:                               
#       message2send4xpu = "<copy4xi>copy2mapimage4xpu</copy4xi>"
#       channel4map2imagepublish4xpu.send(message2send4xpu)
#       print("(Step 93B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel4map2imagepublish4xpu, " MESSAGE2SEND4XPU: " , message2send4xpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4XPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 94A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4xpu(queue4map2image4xpu, channel4map2imageconsume4xpu, channel4map2imagepublish4xpu)
#           print("(Step 94B/700) OF XPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4XPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())      
#   try:
#       message2receive4xpu = channel4map2imageconsume4xpu.recv()
#       print("(Step 95B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel4map2imageconsume4xpu, " CONTROL & COMMAND: ", message2receive4xpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4XPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4XPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2send4xpu = "<ready4xi>ready2mapimage4xpu</ready4xi>"
#       channel4map2imagepublish4xpu.send(message2send4xpu)
#       print("(Step 96B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel4map2imagepublish4xpu, " MESSAGE2SEND4XPU: " , message2send4xpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4XPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 97A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4xpu(queue4map2image4xpu, channel4map2imageconsume4xpu, channel4map2imagepublish4xpu)
#           print("(Step 97B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", manoeuvre2copyimage4xpu)
#       except:
#           print("(Step 97Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4XPU ', time4xi())     
#   try:
#       message2receive4xpu = channel4map2imageconsume4xpu.recv()
#       print("(Step 98B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel4map2imageconsume4xpu, " CONTROL & COMMAND: ", message2receive4xpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4XPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE XPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF XPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE XPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF XPU2IMAGE4XI5TEST - FILENO: ", file2xpu.fileno( ))  
#       print("(Step 99C/700) OF XPU2IMAGE4XI5TEST - MODE: ", file2xpu.mode)
#       print("(Step 99D/700) OF XPU2IMAGE4XI5TEST - NAME: ", file2xpu.name)
#       offset2xpu = 10
#       file2xpu.seek(offset2xpu)
#       print("(Step 99E/700) OF XPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2xpu)
#       file2xpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4XPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 101A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4xpu(queue4map2image4xpu, channel4map2imageconsume4xpu, channel4map2imagepublish4xpu)
#           print("(Step 101B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", manoeuvre2copyimage4xpu)
#       except:
#           print("(Step 101Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4xpu = channel4map2imageconsume4xpu.recv()
#       print("(Step 102B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel4map2imageconsume4xpu, " CONTROL & COMMAND: ", message2receive4xpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4XPU  - LET MANOEUVRE2COPYIMAGE4XPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 103A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4xpu(queue4map2image4xpu, channel4map2imageconsume4xpu, channel4map2imagepublish4xpu)
#           print("(Step 103B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4xpu(task4imagemap2xpu=None):

    '''
    maps 2D images for XPU
    '''
    global xpu2image4xi                                                                                             # 2D-IMAGES FOR XPU  
    global control2mapimage4xpu                                                                                     # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4xpu"
    method2print = "map2image4xpu"

    message2print = "{(Step 1A/33) started" + "; map2image4xpu-started: " + str(map2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2xpu-to-map2image4xpu: " + str(task4imagemap2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) created" + "; xpu2image4xi-created-by-map2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-map2image4xpu: " + str(map2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-map2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; xpu2image4xi-yielded-by-map2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2xpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2xpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2xpu-created-by-map2image4xpu to: " + str(task4imagemap2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                              # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-map2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4xpu"                                                          # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4xpu"                                                          # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2xpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2xpu-created-by-map2image4xpu to: " + str(task4imagemap2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2xpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-map2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-map2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-map2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2xpu ['CONTEXT'] = context2xpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-map2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-map2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-map2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2xpu ['DEVICE'] = device2xpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2xpu-trying-by-map2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2xpu-got-by-map2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2xpu-failed-by-map2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2xpu ['IMAGE'] = imageinfo2xpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2xpu-trying-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2xpu-got-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2xpu-failed-by-map2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2xpu ['QUEUE'] = queue2xpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2xpu-trying-by-map2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2xpu-got-by-map2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2xpu-failed-by-map2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2xpu ['PIPE'] = pipe2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2xpu-trying-by-map2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2xpu-got-by-map2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2xpu-failed-by-map2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2xpu-trying-by-map2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                                   # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2xpu-got-by-map2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2xpu-failed-by-map2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2xpu ['BUFFER'] = bufferinfo2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2xpu-trying-by-map2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2xpu-got-by-map2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2xpu-failed-by-map2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2xpu ['EVENT'] = eventinfo2xpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2xpu-trying-by-map2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2xpu-set-by-map2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2xpu-failed-by-map2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-map2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2xpu-set-by-map2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-map2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2xpu-trying-by-map2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2xpu-set-by-map2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2xpu-failed-by-map2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2xpu ['BLOCKING'] = blocking2xpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2xpu-trying-by-map2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2xpu-set-by-map2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2xpu-failed-by-map2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2xpu ['ORIGIN'] = origin2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2xpu-trying-by-map2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2xpu-set-by-map2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2xpu-failed-by-map2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2xpu ['REGION'] = region2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2xpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2xpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2xpu-trying-by-map2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2xpu-set-by-map2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2xpu-failed-by-map2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2xpu ['ROWPITCH'] = rowpitch2xpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2xpu-trying-by-map2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2xpu-set-by-map2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2xpu-failed-by-map2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2xpu ['SLICEPITCH'] = slicepitch2xpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2xpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2xpu-trying-by-map2image4xpu: " + str(flags2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2xpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2xpu-set-by-map2image4xpu to: " + str(flags2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2xpu-failed-by-map2image4xpu: " + str(flags2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2xpu ['FLAGS'] = flags2xpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2xpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                              # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-map2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4xpu"                                                          # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4xpu"                                                          # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2xpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2xpu-created-by-map2image4xpu to: " + str(task4imagemap2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2xpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-map2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-map2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-map2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2xpu ['CONTEXT'] = context2xpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-map2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-map2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-map2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2xpu ['DEVICE'] = device2xpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2xpu-trying-by-map2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2xpu-got-by-map2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2xpu-failed-by-map2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2xpu ['IMAGE'] = imageinfo2xpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2xpu-trying-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2xpu-got-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2xpu-failed-by-map2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2xpu ['QUEUE'] = queue2xpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2xpu-trying-by-map2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2xpu-got-by-map2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2xpu-failed-by-map2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2xpu ['PIPE'] = pipe2xpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2xpu-trying-by-map2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2xpu-got-by-map2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2xpu-failed-by-map2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2xpu-trying-by-map2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                                   # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2xpu-got-by-map2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2xpu-failed-by-map2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2xpu ['BUFFER'] = bufferinfo2xpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2xpu-trying-by-map2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2xpu-got-by-map2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2xpu-failed-by-map2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2xpu ['EVENT'] = eventinfo2xpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2xpu-trying-by-map2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2xpu-set-by-map2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2xpu-failed-by-map2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-map2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2xpu-set-by-map2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-map2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2xpu-trying-by-map2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2xpu-set-by-map2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2xpu-failed-by-map2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2xpu ['BLOCKING'] = blocking2xpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2xpu-trying-by-map2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2xpu-set-by-map2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2xpu-failed-by-map2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2xpu ['ORIGIN'] = origin2xpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2xpu-trying-by-map2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2xpu-set-by-map2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2xpu-failed-by-map2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2xpu ['REGION'] = region2xpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2xpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2xpu-trying-by-map2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2xpu-set-by-map2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2xpu-failed-by-map2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2xpu ['ROWPITCH'] = rowpitch2xpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2xpu-trying-by-map2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2xpu-set-by-map2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2xpu-failed-by-map2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2xpu ['SLICEPITCH'] = slicepitch2xpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2xpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2xpu-trying-by-map2image4xpu: " + str(flags2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2xpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2xpu-set-by-map2image4xpu to: " + str(flags2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2xpu-failed-by-map2image4xpu: " + str(flags2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2xpu ['FLAGS'] = flags2xpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2xpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2xpu-yielded-by-map2image4xpu: " + str(task4imagemap2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-map2image4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2xpu, publisherchannel2xpu = task4imagemap2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2xpu-got-by-map2image4xpu: " + str(consumerchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2xpu-got-by-map2image4xpu: " + str(publisherchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2xpu = True
        
        control2mapimage4xpu['CONSUMERCHANNEL']  = consumerchannel2xpu
        
        
        control2mapimage4xpu['PUBLISHERCHANNEL'] = publisherchannel2xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2xpu = False
    queue2xpu = None
    message2print = "{(Step 41A/70) trying" + "queue2xpu-trying-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2xpu = task4imagemap2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2xpu-got-by-map2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2xpu = True
        control2mapimage4xpu['QUEUE']  = queue2xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-map2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2mapimage4xpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2xpu, publisherchannel2xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-map2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4xpu-returning-by-map2image4xpu: " + str(control2mapimage4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4xpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4xpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4XPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 42A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4xpu(queue2copy4xpu, channel2copyconsume4xpu, channel2copypublish4xpu)
#           print("(Step 42B/700) OF XPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4XPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4XPU': '<roger4xi>roger2copy4xpu</roger4xi>', 'ACK2INFO4XPU': False, 'MESSAGE2GET4XPU': '<default4xi>default2copy4xpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copy4xpu</copy4xi>', 
#           'CODE2ACK4XPU': ' 0D-1C-', 'MESSAGE2PUT4XPU': '<default4xi>default2copy4xpu</default4xi>', 'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 1}
#        except:
#           print("(Step 42Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4xpu(queue2copymanoeuvre4xpu, consumerchannel2copymanoeuvre4xpu, publisherchannel2copymanoeuvre4xpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4xpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4xpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4xpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4xpu                                 
    global message2copyimageput4xpu
    global message2copyimagereceive4xpu
    global message2copyimagesend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4xpu" 
    method2print = "manoeuvre2copyimage4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4xpu-started: " + str(manoeuvre2copyimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4xpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4xpu-to-manoeuvre2copyimage4xpu: " + str(queue2copymanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4xpu-to-manoeuvre2copyimage4xpu: " + str(consumerchannel2copymanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4xpu-to-manoeuvre2copyimage4xpu: " + str(publisherchannel2copymanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4xpu-try-by-manoeuvre2copyimage4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4xpu is None:
            state2copyimage4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(state2copyimage4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4xpu-used-by-manoeuvre2copyimage4xpu: " + str(state2copyimage4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(state2copyimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(ack2copyimageinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4xpu     = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(message2copyimageget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimageput4xpu     = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(message2copyimageput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimagereceive4xpu = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(message2copyimagereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimagesend4xpu    = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        max2copyimagestate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4xpu-initialized-by-manoeuvre2copyimage4xpu to: " + str( max2copyimagestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4xpu = state2copyimage4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4xpu-added-by-manoeuvre2copyimage4xpu: " + str(state2copyimage4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4xpu > max2copyimagestate4xpu:
        state2copyimage4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(state2copyimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(ack2copyimageinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4xpu     = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(message2copyimageget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimageput4xpu     = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(message2copyimageput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimagereceive4xpu = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(message2copyimagereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        message2copyimagesend4xpu    = '<default4xi>default2copy4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4xpu</default4xi>
        max2copyimagestate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4xpu-set-by-manoeuvre2copyimage4xpu to: " + str( max2copyimagestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4xpu = consumerchannel2copymanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4xpu-received-by-manoeuvre2copyimage4xpu: " + str(message2copyimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4xpu</copy4xi>
                         
            if message2copyimagereceive4xpu == '<copy4xi>copy2copy4xpu</copy4xi>':
                message2copyimagesend4xpu = '<roger4xi>roger2copy4xpu</roger4xi>'
            else:
                message2copyimagesend4xpu = '<negative4xi><negative2copy4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4xpu.send(message2copyimagesend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4xpu-sent-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4xpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(message2copyimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4xpu = consumerchannel2copymanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4xpu-received-by-manoeuvre2copyimage4xpu: " + str(message2copyimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4xpu</ready4xi>
                         
            if message2copyimagereceive4xpu == '<ready4xi>ready2copy4xpu</ready4xi>':
                message2copyimagesend4xpu = '<ready4xi>ready2copy4xpu</ready4xi>'
            else:
                message2copyimagesend4xpu = '<negative4xi><negative2copy4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4xpu.send(message2copyimagesend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4xpu-sent-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(message2copyimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2copyimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2copymanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2copyimage4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(queue2copymanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4xpu is True:
                message2copyimagesend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2copyimagesend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4xpu-set-by-manoeuvre2copyimage4xpu to: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4xpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4xpu-calling-by-manoeuvre2copyimage4xpu: " + str(publisherchannel2copymanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4xpu-passing-to-send-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4xpu.send(message2copyimagesend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4xpu-sent-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4xpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4xpu-failed-by-manoeuvre2copyimage4xpu: " + str(message2copyimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4XPU']        = ack2copyimageinfo4xpu
    response2manouvre4copy['CODE2ACK4XPU']        = code2ack
    response2manouvre4copy['CODE2NACK4XPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4XPU']     = message2copyimageget4xpu
    response2manouvre4copy['MESSAGE2PUT4XPU']     = message2copyimageput4xpu
    response2manouvre4copy['MESSAGE2RECEIVE4XPU'] = message2copyimagereceive4xpu
    response2manouvre4copy['MESSAGE2SEND4XPU']    = message2copyimagesend4xpu
    response2manouvre4copy['STATE2MANOEUVRE4XPU'] = state2copyimage4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4xpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4XPU': '<roger4xi>roger2copy4xpu</roger4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copy4xpu</copy4xi>',
    #   'CODE2ACK4XPU': ' 0D-1C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4XPU': '<ready4xi>ready2copy4xpu</ready4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copy4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-2C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>',
    #   'ACK2INFO4XPU': True,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copy4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'CODE2ACK4XPU': ' 0D-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copy4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4xpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4XPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 62A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4xpu(queue4copy2imagetobuffer4xpu, channel4copy2imagetobufferconsume4xpu, channel4copy2imagetobufferpublish4xpu)
#           print("(Step 62B/700) OF XPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4XPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4XPU': '<roger4xi>roger2copyimagetobuffer4xpu</roger4xi>',
#           #   'ACK2INFO4XPU': False,
#           #   'MESSAGE2GET4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
#           #   'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copyimagetobuffer4xpu</copy4xi>',
#           #   'CODE2ACK4XPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
#           #   'CODE2NACK4XPU': ' ',
#           #   'STATE2MANOEUVRE4XPU': 1}
#        except:
#           print("(Step 62Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4xpu(queue2copyimagetobuffermanoeuvre4xpu, consumerchannel2copyimagetobuffermanoeuvre4xpu, publisherchannel2copyimagetobuffermanoeuvre4xpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4xpu
    '''
    global state2copyimagetobuffer4xpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4xpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4xpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4xpu                                 
    global message2copyimagetobufferput4xpu
    global message2copyimagetobufferreceive4xpu
    global message2copyimagetobuffersend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4xpu" 
    method2print = "manoeuvre2copyimagetobuffer4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4xpu-started: " + str(manoeuvre2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4xpu-to-manoeuvre2copyimagetobuffer4xpu: " + str(queue2copyimagetobuffermanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4xpu-to-manoeuvre2copyimagetobuffer4xpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4xpu-to-manoeuvre2copyimagetobuffer4xpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4xpu-try-by-manoeuvre2copyimagetobuffer4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4xpu is None:
            state2copyimagetobuffer4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(state2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4xpu-used-by-manoeuvre2copyimagetobuffer4xpu: " + str(state2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(state2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(ack2copyimagetobufferinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4xpu     = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4xpu     = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4xpu = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4xpu    = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4xpu-initialized-by-manoeuvre2copyimagetobuffer4xpu to: " + str( max2copyimagetobufferstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4xpu = state2copyimagetobuffer4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4xpu-added-by-manoeuvre2copyimagetobuffer4xpu: " + str(state2copyimagetobuffer4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4xpu > max2copyimagetobufferstate4xpu:
        state2copyimagetobuffer4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(state2copyimagetobuffer4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(ack2copyimagetobufferinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4xpu     = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4xpu     = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4xpu = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobufferreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4xpu    = '<default4xi>default2copyimagetobuffer4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str( max2copyimagetobufferstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4xpu = consumerchannel2copyimagetobuffermanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4xpu-received-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobufferreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4xpu == '<copy4xi>copy2copyimagetobuffer4xpu</copy4xi>': 
                message2copyimagetobuffersend4xpu = '<roger4xi>roger2copyimagetobuffer4xpu</roger4xi>'
            else:
                message2copyimagetobuffersend4xpu = '<negative4xi><negative2copyimagetobuffer4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4xpu.send(message2copyimagetobuffersend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4xpu-sent-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobufferreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4xpu = consumerchannel2copyimagetobuffermanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4xpu-received-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobufferreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4xpu == '<ready4xi>ready2copyimagetobuffer4xpu</ready4xi>':
                message2copyimagetobuffersend4xpu = '<ready4xi>ready2copyimagetobuffer4xpu</ready4xi>'
            else:
                message2copyimagetobuffersend4xpu = '<negative4xi><negative2copyimagetobuffer4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4xpu.send(message2copyimagetobuffersend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4xpu-sent-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobufferreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2copyimagetobuffer4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2copyimagetobuffermanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2copyimagetobuffer4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(queue2copyimagetobuffermanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4xpu is True:
                message2copyimagetobuffersend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2copyimagetobuffersend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4xpu-set-by-manoeuvre2copyimagetobuffer4xpu to: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4xpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4xpu-calling-by-manoeuvre2copyimagetobuffer4xpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4xpu-passing-to-send-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4xpu.send(message2copyimagetobuffersend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4xpu-sent-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4xpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4xpu-failed-by-manoeuvre2copyimagetobuffer4xpu: " + str(message2copyimagetobuffersend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4XPU']        = ack2copyimagetobufferinfo4xpu
    response2manouvre4copyimagetobuffer['CODE2ACK4XPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4XPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4XPU']     = message2copyimagetobufferget4xpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4XPU']     = message2copyimagetobufferput4xpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4XPU'] = message2copyimagetobufferreceive4xpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4XPU']    = message2copyimagetobuffersend4xpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4XPU'] = state2copyimagetobuffer4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4xpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4XPU': '<roger4xi>roger2copyimagetobuffer4xpu</roger4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copyimagetobuffer4xpu</copy4xi>',
    #   'CODE2ACK4XPU': ' 0D-1C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4XPU': '<ready4xi>ready2copyimagetobuffer4xpu</ready4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copyimagetobuffer4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-2C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>',
    #   'ACK2INFO4XPU': True,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copyimagetobuffer4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'CODE2ACK4XPU': ' 0D-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copyimagetobuffer4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4xpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4XPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4xpu is True:
#       print("(Step 82A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4xpu(queue4copybuffertoimage4xpu, channel4copybuffertoimageconsume4xpu, channel4copybuffertoimagepublish4xpu)
#           print("(Step 82B/700) OF XPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4XPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4XPU': '<roger4xi>roger2copybuffertoimage4xpu</roger4xi>',
#           #   'ACK2INFO4XPU': False,
#           #   'MESSAGE2GET4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
#           #   'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copybuffertoimage4xpu</copy4xi>',
#           #   'CODE2ACK4XPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
#           #   'CODE2NACK4XPU': ' ',
#           #   'STATE2MANOEUVRE4XPU': 1}
#       except:
#           print("(Step 82Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4xpu(queue2copybuffertoimagemanoeuvre4xpu, consumerchannel2copybuffertoimagemanoeuvre4xpu, publisherchannel2copybuffertoimagemanoeuvre4xpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4xpu
    '''
    global state2copybuffertoimage4xpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4xpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4xpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4xpu                                 
    global message2copybuffertoimageput4xpu
    global message2copybuffertoimagereceive4xpu
    global message2copybuffertoimagesend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4xpu" 
    method2print = "manoeuvre2copybuffertoimage4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4xpu-started: " + str(manoeuvre2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4xpu-to-manoeuvre2copybuffertoimage4xpu: " + str(queue2copybuffertoimagemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4xpu-to-manoeuvre2copybuffertoimage4xpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4xpu-to-manoeuvre2copybuffertoimage4xpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4xpu-try-by-manoeuvre2copybuffertoimage4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4xpu is None:
            state2copybuffertoimage4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(state2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4xpu-used-by-manoeuvre2copybuffertoimage4xpu: " + str(state2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(state2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(ack2copybuffertoimageinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4xpu     = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimageget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4xpu     = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimageput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4xpu = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimagereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4xpu    = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4xpu-initialized-by-manoeuvre2copybuffertoimage4xpu to: " + str( max2copybuffertoimagestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4xpu = state2copybuffertoimage4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4xpu-added-by-manoeuvre2copybuffertoimage4xpu: " + str(state2copybuffertoimage4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4xpu > max2copybuffertoimagestate4xpu:
        state2copybuffertoimage4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(state2copybuffertoimage4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(ack2copybuffertoimageinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4xpu     = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimageget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4xpu     = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimageput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4xpu = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimagereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4xpu    = '<default4xi>default2copybuffertoimage4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str( max2copybuffertoimagestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4xpu = consumerchannel2copybuffertoimagemanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4xpu-received-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4xpu == '<copy4xi>copy2copybuffertoimage4xpu</copy4xi>':
                message2copybuffertoimagesend4xpu = '<roger4xi>roger2copybuffertoimage4xpu</roger4xi>'
            else:
                message2copybuffertoimagesend4xpu = '<negative4xi><negative2copybuffertoimage4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4xpu.send(message2copybuffertoimagesend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4xpu-sent-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4xpu = consumerchannel2copybuffertoimagemanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4xpu-received-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4xpu == '<ready4xi>ready2copybuffertoimage4xpu</ready4xi>':
                message2copybuffertoimagesend4xpu = '<ready4xi>ready2copybuffertoimage4xpu</ready4xi>'
            else:
                message2copybuffertoimagesend4xpu = '<negative4xi><negative2copybuffertoimage4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4xpu.send(message2copybuffertoimagesend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4xpu-sent-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2copybuffertoimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2copybuffertoimagemanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2copybuffertoimage4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(queue2copybuffertoimagemanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4xpu is True:
                message2copybuffertoimagesend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2copybuffertoimagesend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4xpu-set-by-manoeuvre2copybuffertoimage4xpu to: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4xpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4xpu-calling-by-manoeuvre2copybuffertoimage4xpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4xpu-passing-to-send-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4xpu.send(message2copybuffertoimagesend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4xpu-sent-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4xpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4xpu-failed-by-manoeuvre2copybuffertoimage4xpu: " + str(message2copybuffertoimagesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4XPU']        = ack2copybuffertoimageinfo4xpu
    response2manouvre4copybuffertoimage['CODE2ACK4XPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4XPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4XPU']     = message2copybuffertoimageget4xpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4XPU']     = message2copybuffertoimageput4xpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4XPU'] = message2copybuffertoimagereceive4xpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4XPU']    = message2copybuffertoimagesend4xpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4XPU'] = state2copybuffertoimage4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4xpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4XPU': '<roger4xi>roger2copybuffertoimage4xpu</roger4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2copybuffertoimage4xpu</copy4xi>',
    #   'CODE2ACK4XPU': ' 0D-1C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4XPU': '<ready4xi>ready2copybuffertoimage4xpu</ready4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copybuffertoimage4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-2C-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>',
    #   'ACK2INFO4XPU': True,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2copybuffertoimage4xpu</ready4xi>',
    #   'CODE2ACK4XPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'ACK2INFO4XPU': False,
    #   'MESSAGE2GET4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'MESSAGE2RECEIVE4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'CODE2ACK4XPU': ' 0D-',
    #   'MESSAGE2PUT4XPU': '<default4xi>default2copybuffertoimage4xpu</default4xi>',
    #   'CODE2NACK4XPU': ' ',
    #   'STATE2MANOEUVRE4XPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4xpu" usage = "control function for read2image4xpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4xpu(queue2readmanoeuvre4xpu, consumerchannel2readmanoeuvre4xpu, publisherchannel2readmanoeuvre4xpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4xpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4xpu                                                                                        # MAX FOR STATE
    global ack2readinfo4xpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4xpu                                 
    global message2readput4xpu
    global message2readreceive4xpu
    global message2readsend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4xpu" 
    method2print = "manoeuvre2readimage4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4xpu-started: " + str(manoeuvre2readimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4xpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4xpu-to-manoeuvre2readimage4xpu: " + str(queue2readmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4xpu-to-manoeuvre2readimage4xpu: " + str(consumerchannel2readmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4xpu-to-manoeuvre2readimage4xpu: " + str(publisherchannel2readmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4xpu-try-by-manoeuvre2readimage4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4xpu is None:
            state2read4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(state2read4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4xpu-used-by-manoeuvre2readimage4xpu: " + str(state2read4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(state2read4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(ack2readinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4xpu     = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(message2readget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4xpu</default4xi>
        message2readput4xpu     = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(message2readput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        message2readreceive4xpu = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(message2readreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        message2readsend4xpu    = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        max2readstate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4xpu-initialized-by-manoeuvre2readimage4xpu to: " + str( max2readstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4xpu = state2read4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4xpu-added-by-manoeuvre2readimage4xpu: " + str(state2read4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4xpu > max2readstate4xpu:
        state2read4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4xpu-set-by-manoeuvre2readimage4xpu to: " + str(state2read4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4xpu-set-by-manoeuvre2readimage4xpu to: " + str(ack2readinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4xpu     = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4xpu-set-by-manoeuvre2readimage4xpu to: " + str(message2readget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4xpu</default4xi>
        message2readput4xpu     = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4xpu-set-by-manoeuvre2readimage4xpu to: " + str(message2readput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        message2readreceive4xpu = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4xpu-set-by-manoeuvre2readimage4xpu to: " + str(message2readreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        message2readsend4xpu    = '<default4xi>default2read4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4xpu-set-by-manoeuvre2readimage4xpu to: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4xpu</default4xi>
        max2readstate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4xpu-set-by-manoeuvre2readimage4xpu to: " + str( max2readstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4xpu = consumerchannel2readmanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4xpu-received-by-manoeuvre2readimage4xpu: " + str(message2readreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4xpu</copy4xi>
                         
            if message2readreceive4xpu == '<copy4xi>copy2read4xpu</copy4xi>':
                message2readsend4xpu = '<roger4xi>roger2read4xpu</roger4xi>'
            else:
                message2readsend4xpu = '<negative4xi><negative2read4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4xpu.send(message2readsend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4xpu-sent-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4xpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4xpu-failed-by-manoeuvre2readimage4xpu: " + str(message2readreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4xpu = consumerchannel2readmanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4xpu-received-by-manoeuvre2readimage4xpu: " + str(message2readreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4xpu</ready4xi> 
                         
            if message2readreceive4xpu == '<ready4xi>ready2read4xpu</ready4xi>':
                message2readsend4xpu = '<ready4xi>ready2read4xpu</ready4xi>'
            else:
                message2readsend4xpu = '<negative4xi><negative2ready4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4xpu.send(message2readsend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4xpu-sent-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4xpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4xpu-failed-by-manoeuvre2readimage4xpu: " + str(message2readreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2readimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2readmanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2readimage4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4xpu-failed-by-manoeuvre2readimage4xpu: " + str(queue2readmanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4xpu is True:
                message2readsend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2readsend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4xpu-set-by-manoeuvre2readimage4xpu to: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4xpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4xpu-failed-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4xpu-calling-by-manoeuvre2readimage4xpu: " + str(publisherchannel2readmanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4xpu-passing-to-send-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4xpu-failed-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4xpu.send(message2readsend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4xpu-sent-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4xpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4xpu-failed-by-manoeuvre2readimage4xpu: " + str(message2readsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4XPU']        = ack2readinfo4xpu
    response2manouvre4read['CODE2ACK4XPU']        = code2ack
    response2manouvre4read['CODE2NACK4XPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4XPU']     = message2readget4xpu
    response2manouvre4read['MESSAGE2PUT4XPU']     = message2readput4xpu
    response2manouvre4read['MESSAGE2RECEIVE4XPU'] = message2readreceive4xpu
    response2manouvre4read['MESSAGE2SEND4XPU']    = message2readsend4xpu
    response2manouvre4read['STATE2MANOEUVRE4XPU'] = state2read4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4xpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4XPU': '<roger4xi>roger2read4xpu</roger4xi>',
    #  'ACK2INFO4XPU': False,
    #  'MESSAGE2GET4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2read4xpu</copy4xi>',
    #  'CODE2ACK4XPU': ' 0D-1C-',
    #  'MESSAGE2PUT4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4XPU': '<ready4xi>ready2read4xpu</ready4xi>',
    #  'ACK2INFO4XPU': False,
    #  'MESSAGE2GET4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2read4xpu</ready4xi>',
    #  'CODE2ACK4XPU': ' 0D-2C-',
    #  'MESSAGE2PUT4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>',
    #  'ACK2INFO4XPU': True,
    #  'MESSAGE2GET4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2read4xpu</ready4xi>',
    #  'CODE2ACK4XPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'CODE2NACK4XPU': ' ',
    #  'STATE2MANOEUVRE4XPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'ACK2INFO4XPU': False,
    #  'MESSAGE2GET4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'MESSAGE2RECEIVE4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'CODE2ACK4XPU': ' 0D-',
    #  'MESSAGE2PUT4XPU': '<default4xi>default2read4xpu</default4xi>',
    #  'CODE2NACK4XPU': ' ',
    #  'STATE2MANOEUVRE4XPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4xpu" usage = "control function for write2image4xpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4xpu(queue2writemanoeuvre4xpu, consumerchannel2writemanoeuvre4xpu, publisherchannel2writemanoeuvre4xpu):
    '''
    manoeuvres control and command
    '''
    global state2write4xpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4xpu                                                    # MAX FOR STATE
    global ack2writeinfo4xpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4xpu                                 
    global message2writeput4xpu
    global message2writereceive4xpu
    global message2writesend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4xpu" 
    method2print = "manoeuvre2writeimage4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4xpu-started: " + str(manoeuvre2writeimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4xpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4xpu-to-manoeuvre2writeimage4xpu: " + str(queue2writemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4xpu-to-manoeuvre2writeimage4xpu: " + str(consumerchannel2writemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4xpu-to-manoeuvre2writeimage4xpu: " + str(publisherchannel2writemanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4xpu-try-by-manoeuvre2writeimage4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4xpu is None:
            state2write4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(state2write4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4xpu-used-by-manoeuvre2writeimage4xpu: " + str(state2write4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(state2write4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(ack2writeinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4xpu     = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(message2writeget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4xpu</default4xi>
        message2writeput4xpu     = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(message2writeput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        message2writereceive4xpu = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(message2writereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        message2writesend4xpu    = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        max2writestate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4xpu-initialized-by-manoeuvre2writeimage4xpu to: " + str( max2writestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4xpu = state2write4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4xpu-added-by-manoeuvre2writeimage4xpu: " + str(state2write4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4xpu > max2writestate4xpu:
        state2write4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(state2write4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(ack2writeinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4xpu     = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(message2writeget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4xpu</default4xi>
        message2writeput4xpu     = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(message2writeput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        message2writereceive4xpu = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(message2writereceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        message2writesend4xpu    = '<default4xi>default2write4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4xpu</default4xi>
        max2writestate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4xpu-set-by-manoeuvre2writeimage4xpu to: " + str( max2writestate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4xpu = consumerchannel2writemanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4xpu-received-by-manoeuvre2writeimage4xpu: " + str(message2writereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4xpu</copy4xi>
                         
            if message2writereceive4xpu == '<copy4xi>copy2write4xpu</copy4xi>':
                message2writesend4xpu = '<roger4xi>roger2write4xpu</roger4xi>'
            else:
                message2writesend4xpu = '<negative4xi><negative2write4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4xpu.send(message2writesend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4xpu-sent-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4xpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(message2writereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4xpu = consumerchannel2writemanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4xpu-received-by-manoeuvre2writeimage4xpu: " + str(message2writereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4xpu</ready4xi>
                         
            if message2writereceive4xpu == '<ready4xi>ready2write4xpu</ready4xi>':
                message2writesend4xpu = '<ready4xi>ready2write4xpu</ready4xi>'
            else:
                message2writesend4xpu = '<negative4xi><negative2ready4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4xpu.send(message2writesend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4xpu-sent-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4xpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(message2writereceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2writeimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2writemanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2writeimage4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(queue2writemanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4xpu is True:
                message2writesend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2writesend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4xpu-set-by-manoeuvre2writeimage4xpu to: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4xpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4xpu-calling-by-manoeuvre2writeimage4xpu: " + str(publisherchannel2writemanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4xpu-passing-to-send-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4xpu.send(message2writesend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4xpu-sent-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4xpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4xpu-failed-by-manoeuvre2writeimage4xpu: " + str(message2writesend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4XPU']        = ack2writeinfo4xpu
    response2manouvre4write['CODE2ACK4XPU']        = code2ack
    response2manouvre4write['CODE2NACK4XPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4XPU']     = message2writeget4xpu
    response2manouvre4write['MESSAGE2PUT4XPU']     = message2writeput4xpu
    response2manouvre4write['MESSAGE2RECEIVE4XPU'] = message2writereceive4xpu
    response2manouvre4write['MESSAGE2SEND4XPU']    = message2writesend4xpu
    response2manouvre4write['STATE2MANOEUVRE4XPU'] = state2write4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4xpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4XPU': '<roger4xi>roger2write4xpu</roger4xi>', 
    #  'ACK2INFO4XPU': False, 'MESSAGE2GET4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2write4xpu</copy4xi>', 
    #  'CODE2ACK4XPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4XPU': '<ready4xi>ready2write4xpu</ready4xi>', 
    #  'ACK2INFO4XPU': False, 'MESSAGE2GET4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2write4xpu</ready4xi>', 
    #  'CODE2ACK4XPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>', 
    #  'ACK2INFO4XPU': True, 'MESSAGE2GET4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2write4xpu</ready4xi>', 
    #  'CODE2ACK4XPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2write4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4xpu" usage = "control function for map2image4xpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4xpu(queue2mapmanoeuvre4xpu, consumerchannel2mapmanoeuvre4xpu, publisherchannel2mapmanoeuvre4xpu):
    '''
    manoeuvres control and command
    '''
    global state2map4xpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4xpu                                                    # MAX FOR STATE
    global ack2mapinfo4xpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4xpu                                 
    global message2mapput4xpu
    global message2mapreceive4xpu
    global message2mapsend4xpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4xpu" 
    method2print = "manoeuvre2mapimage4xpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4xpu-started: " + str(manoeuvre2mapimage4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4xpu-to-manoeuvre2mapimage4xpu: " + str(queue2mapmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4xpu-to-manoeuvre2mapimage4xpu: " + str(consumerchannel2mapmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4xpu-to-manoeuvre2mapimage4xpu: " + str(publisherchannel2mapmanoeuvre4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4xpu-try-by-manoeuvre2mapimage4xpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4xpu is None:
            state2map4xpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(state2map4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4xpu-used-by-manoeuvre2mapimage4xpu: " + str(state2map4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4xpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(state2map4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4xpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(ack2mapinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4xpu     = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(message2mapget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4xpu</default4xi>
        message2mapput4xpu     = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(message2mapput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        message2mapreceive4xpu = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(message2mapreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        message2mapsend4xpu    = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        max2mapstate4xpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4xpu-initialized-by-manoeuvre2mapimage4xpu to: " + str( max2mapstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4xpu = state2map4xpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4xpu-added-by-manoeuvre2mapimage4xpu: " + str(state2map4xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4xpu > max2mapstate4xpu:
        state2map4xpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(state2map4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4xpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(ack2mapinfo4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4xpu     = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(message2mapget4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4xpu</default4xi>
        message2mapput4xpu     = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(message2mapput4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        message2mapreceive4xpu = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(message2mapreceive4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        message2mapsend4xpu    = '<default4xi>default2map4xpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4xpu</default4xi>
        max2mapstate4xpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4xpu-set-by-manoeuvre2mapimage4xpu to: " + str( max2mapstate4xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4xpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4xpu = consumerchannel2mapmanoeuvre4xpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4xpu-received-by-manoeuvre2mapimage4xpu: " + str(message2mapreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4xpu == '<copy4xi>copy2mapimage4xpu</copy4xi>':
                message2mapsend4xpu = '<roger4xi>roger2mapimage4xpu</roger4xi>'
            else:
                message2mapsend4xpu = '<negative4xi><negative2mapimage4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4xpu.send(message2mapsend4xpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4xpu-sent-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(message2mapreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4xpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4xpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4xpu = consumerchannel2mapmanoeuvre4xpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4xpu-received-by-manoeuvre2mapimage4xpu: " + str(message2mapreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4xpu == '<ready4xi>ready2mapimage4xpu</ready4xi>':
                message2mapsend4xpu = '<ready4xi>ready2mapimage4xpu</ready4xi>'
            else:
                message2mapsend4xpu = '<negative4xi><negative2ready4xpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4xpu.send(message2mapsend4xpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4xpu-sent-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(message2mapreceive4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4xpu == 3:    
        info2image4xpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4xpu-try-by-manoeuvre2mapimage4xpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4xpu = queue2mapmanoeuvre4xpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4xpu-got-by-manoeuvre2mapimage4xpu: " + str(info2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4xpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4xpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4xpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(queue2mapmanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4xpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4xpu is True:
                message2mapsend4xpu = '<ack4xi>ack2imageinfo4xpu</ack4xi>'
            else:
                message2mapsend4xpu = '<nack4xi>nack2imageinfo4xpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4xpu-set-by-manoeuvre2mapimage4xpu to: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4xpu-calling-by-manoeuvre2mapimage4xpu: " + str(publisherchannel2mapmanoeuvre4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4xpu-passing-to-send-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4xpu.send(message2mapsend4xpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4xpu-sent-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4xpu-failed-by-manoeuvre2mapimage4xpu: " + str(message2mapsend4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4XPU']        = ack2mapinfo4xpu
    response2manouvre4map['CODE2ACK4XPU']        = code2ack
    response2manouvre4map['CODE2NACK4XPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4XPU']     = message2mapget4xpu
    response2manouvre4map['MESSAGE2PUT4XPU']     = message2mapput4xpu
    response2manouvre4map['MESSAGE2RECEIVE4XPU'] = message2mapreceive4xpu
    response2manouvre4map['MESSAGE2SEND4XPU']    = message2mapsend4xpu
    response2manouvre4map['STATE2MANOEUVRE4XPU'] = state2map4xpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4xpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4XPU': '<roger4xi>roger2mapimage4xpu</roger4xi>', 
    #  'ACK2INFO4XPU': False, 
    #  'MESSAGE2GET4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<copy4xi>copy2mapimage4xpu</copy4xi>', 
    #  'CODE2ACK4XPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 1}
     
    #2 {'MESSAGE2SEND4XPU': '<ready4xi>ready2mapimage4xpu</ready4xi>', 
    #  'ACK2INFO4XPU': False, 
    #  'MESSAGE2GET4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2mapimage4xpu</ready4xi>', 
    #  'CODE2ACK4XPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4XPU': '<ack4xi>ack2imageinfo4xpu</ack4xi>', 
    #  'ACK2INFO4XPU': True, 
    #  'MESSAGE2GET4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<ready4xi>ready2mapimage4xpu</ready4xi>', 
    #  'CODE2ACK4XPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 
    #  'STATE2MANOEUVRE4XPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'ACK2INFO4XPU': False, 
    #  'MESSAGE2GET4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'MESSAGE2RECEIVE4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'CODE2ACK4XPU': ' 0D-', 
    #  'MESSAGE2PUT4XPU': '<default4xi>default2map4xpu</default4xi>', 
    #  'CODE2NACK4XPU': ' ', 'STATE2MANOEUVRE4XPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4xpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4xpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4XPU : ', response4write2image)
#       queue2write4xpu      = response4write2image['QUEUE']
#       channel2publish4xpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4xpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4XPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2writesend4xpu = "<copy4xi>copy2write4xpu</copy4xi>"
#       channel2publish4xpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4xpu.send(message2writesend4xpu)
#       print("(Step 17B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2publish4xpu, " MESSAGE2SEND4XPU: " , message2writesend4xpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4XPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2write4xpu, channel2consume4xpu, channel2publish4xpu)
#       print("(Step 18D/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 18Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:
#       channel2consume4xpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4xpu = channel2consume4xpu.recv()
#       print("(Step 19B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2consume4xpu, " CONTROL & COMMAND: ", message2writereceive4xpu)                                                                                                  
#       # <roger4xi>roger2write4xpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4XPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4XPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF XPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:                               
#       message2writesend4xpu = "<write4xi>write2write4xpu</write4xi>"
#       channel2publish4xpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4xpu.send(message2writesend4xpu)
#       print("(Step 20B/700) OF XPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4XPU: ", channel2publish4xpu, " MESSAGE2SEND4XPU: " , message2writesend4xpu)                                                                                                
#       # <write4xi>write2write4xpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4XPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2write4xpu, channel2consume4xpu, channel2publish4xpu)
#       print("(Step 21B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 21Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4XPU ', time4xi())     
#   try:
#       channel2consume4xpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4xpu = channel2consume4xpu.recv()
#       print("(Step 22B/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2consume4xpu, " CONTROL & COMMAND: ", message2writereceive4xpu)                                                                                                  
#       # <write4xi>write2write4xpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4XPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4XPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2write4xpu, channel2consume4xpu, channel2publish4xpu)
#       print("(Step 25B/700) OF XPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", manoeuvre2readimage4xpu)
#   except:
#       print("(Step 25Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4xpu = channel2consume4xpu.recv()
#       print("(Step 26D/700) OF XPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4XPU: ", channel2consume4xpu, " CONTROL & COMMAND: ", message2writereceive4xpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4xpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4XPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF XPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi()) 
#   try:        
#       response2manoeuvre4xpu = manoeuvre2readimage4xpu(queue2write4xpu, channel2consume4xpu, channel2publish4xpu)
#       print("(Step 27B/700) OF XPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4XPU ", response2manoeuvre4xpu)
#   except:
#       print("(Step 27Z/700) OF XPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4XPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4xpu(task4imagewrite2xpu=None):

    '''
    writes 2D images for XPU
    '''
    global xpu2image4xi                                                     # 2D-IMAGES FOR XPU  
    global control2writeimage4xpu                                              # COMMAND AND CONTROL FOR XPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4xpu"
    method2print = "write2image4xpu"

    message2print = "{(Step 1A/33) started" + "; write2image4xpu-started: " + str(write2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4xpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2xpu-to-write2image4xpu: " + str(task4imagewrite2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE XPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                              # INITIALIZE IMAGE FOR XPU 
            message2print = "{(Step 1C/70) created" + "; xpu2image4xi-created-by-write2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4xpu-skipped-by-write2image4xpu: " + str(write2image4xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4xpu-skipped-by-write2image4xpu: <function write2image4xpu at 0x00000000038E8EC8>                                 
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 1E/70) set" + "; xpu2image4xi-set-by-write2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            xpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; xpu2image4xi-yielded-by-write2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2xpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2xpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2xpu-created-by-write2image4xpu to: " + str(task4imagewrite2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                                 # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4xpu-created-by-write2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4xpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4xpu"                                                            # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4xpu"                                                            # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4xpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4xpu', 
            #  'METHOD': 'write2image4xpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2xpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2xpu-created-by-write2image4xpu to: " + str(task4imagewrite2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4xpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4xpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4xpu',
            #  'METHOD': 'write2image4xpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2xpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2xpu-trying-by-write2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2xpu-got-by-write2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4xpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2xpu-failed-by-write2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2xpu ['CONTEXT'] = context2xpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2xpu-trying-by-write2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2xpu-got-by-write2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2xpu-failed-by-write2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2xpu ['DEVICE'] = device2xpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2xpu-trying-by-write2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2xpu-got-by-write2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4xpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4xpu', 
                #  'METHOD': 'create2context4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4xpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4xpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2xpu-failed-by-write2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2xpu ['IMAGE'] = imageinfo2xpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2xpu-trying-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2xpu-got-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2xpu-failed-by-write2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2xpu ['QUEUE'] = queue2xpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2xpu-trying-by-write2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2xpu-got-by-write2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2xpu-failed-by-write2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2xpu ['PIPE'] = pipe2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2xpu-trying-by-write2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2xpu-got-by-write2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4xpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4xpu', 
                #  'METHOD': 'create2commandqueue4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4xpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2xpu-failed-by-write2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2xpu-trying-by-write2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                               # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2xpu-got-by-write2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4xpu', 
                #  'METHOD': 'create2buffer4xpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2xpu-failed-by-write2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2xpu ['BUFFER'] = bufferinfo2xpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2xpu-trying-by-write2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2xpu-got-by-write2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4xpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4xpu', 
                #  'METHOD': 'create2event4xpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4xpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2xpu-failed-by-write2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2xpu ['EVENT'] = eventinfo2xpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2xpu-trying-by-write2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2xpu-set-by-write2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2xpu-failed-by-write2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-write2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2xpu-set-by-write2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-write2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2xpu-trying-by-write2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2xpu-set-by-write2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2xpu-failed-by-write2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2xpu ['BLOCKING'] = blocking2xpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2xpu-trying-by-write2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2xpu-set-by-write2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2xpu-failed-by-write2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2xpu ['ORIGIN'] = origin2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2xpu-trying-by-write2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2xpu-set-by-write2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2xpu-failed-by-write2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2xpu ['REGION'] = region2xpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2xpu = 800
            width2xpu = 800
            rowpitch2xpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2xpu-trying-by-write2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2xpu-set-by-write2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2xpu-failed-by-write2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2xpu ['ROWPITCH'] = rowpitch2xpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2xpu = None
            size2xpu = int(width2xpu * height2xpu)                                                                  # CALCULATED MEMORY
            if size2xpu == 0:
                size2xpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2xpu-trying-by-write2image4xpu: " + str(pointer2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2xpu = Arena4Xi(size2xpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2xpu-got-by-write2image4xpu: " + str(arena2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2xpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4xpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2xpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4xpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2xpu:
                    size2xpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2xpu-set-by-write2buffer4xpu to: " + str(size2xpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2xpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2xpu-set-by-write2buffer4xpu to: " + str(pointer2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2xpu-failed-by-write2image4xpu: " + str(pointer2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2xpu ['POINTER'] = pointer2xpu                            
            task4imagewrite2xpu ['SIZE']    = size2xpu                                                               # AUTOMATIC BY XPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2xpu-trying-by-write2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2xpu-set-by-write2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2xpu-failed-by-write2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2xpu ['SLICEPITCH'] = slicepitch2xpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2xpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4xpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4xpu = trace2image4xpu                                                             # CALLBACK FUNCTION FOR XPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4xpu-created-by-write2image4xpu to: " + str(function2callback4xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4xpu"                                                        # MODULE OF XPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4xpu"                                                        # FUNCTION OF XPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4xpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR XPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4xpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2xpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO XPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2xpu['FUNCTION2CALLBACK'] = function2callback4xpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2xpu-created-by-write2image4xpu to: " + str(task4imagewrite2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4xpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4xpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4xpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2xpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2xpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2xpu-trying-by-write2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2xpu = get2context4xpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2xpu-got-by-write2image4xpu: " + str(context2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2xpu-failed-by-write2image4xpu: " + str(context2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2xpu ['CONTEXT'] = context2xpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2xpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2xpu-trying-by-write2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2xpu = get2xpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2xpu-got-by-write2image4xpu: " + str(device2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2xpu-failed-by-write2image4xpu: " + str(device2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2xpu ['DEVICE'] = device2xpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2xpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2xpu-trying-by-write2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2xpu = get2imageinfo4xpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2xpu-got-by-write2image4xpu: " + str(imageinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2xpu-failed-by-write2image4xpu: " + str(imageinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2xpu ['IMAGE'] = imageinfo2xpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY XPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2xpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2xpu-trying-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2xpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2xpu-got-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2xpu-failed-by-write2image4xpu: " + str(queue2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2xpu ['QUEUE'] = queue2xpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY XPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2xpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2xpu-trying-by-write2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2xpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2xpu-got-by-write2image4xpu: " + str(pipe2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2xpu-failed-by-write2image4xpu: " + str(pipe2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2xpu ['PIPE'] = pipe2xpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2xpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2xpu-trying-by-write2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2xpu = get2commandqueueinfo4xpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2xpu-got-by-write2image4xpu: " + str(commandqueueinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2xpu-failed-by-write2image4xpu: " + str(commandqueueinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2xpu ['COMMANDQUEUE'] = commandqueueinfo2xpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY XPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2xpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2xpu-trying-by-write2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2xpu = get2bufferinfo4xpu()                                                           # SYSTEM CREATES BUFFER FOR XPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2xpu-got-by-write2image4xpu: " + str(bufferinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2xpu-failed-by-write2image4xpu: " + str(bufferinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2xpu ['BUFFER'] = bufferinfo2xpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2xpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2xpu-trying-by-write2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2xpu = get2eventinfo4xpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2xpu-got-by-write2image4xpu: " + str(eventinfo2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2xpu-failed-by-write2image4xpu: " + str(eventinfo2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2xpu ['EVENT'] = eventinfo2xpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2xpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2xpu-trying-by-write2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2xpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2xpu-set-by-write2image4xpu: " + str(eventwaitlist2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2xpu-failed-by-write2image4xpu: " + str(eventwaitlist2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2xpu ['EVENTWAITLIST'] = eventwaitlist2xpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2xpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2xpu-trying-by-write2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2xpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2xpu = eventwaitlistquantity2xpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2xpu-set-by-write2image4xpu: " + str(eventwaitlistquantity2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2xpu-failed-by-write2image4xpu: " + str(eventwaitlistquantity2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2xpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2xpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2xpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2xpu-trying-by-write2image4xpu: " + str(blocking2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2xpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2xpu-set-by-write2image4xpu to: " + str(blocking2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2xpu-failed-by-write2image4xpu: " + str(blocking2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2xpu ['BLOCKING'] = blocking2xpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2xpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2xpu-trying-by-write2image4xpu: " + str(origin2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2xpu['X'] = 0 
                origin2xpu['Y'] = 0  
                origin2xpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2xpu-set-by-write2image4xpu to: " + str(origin2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2xpu-failed-by-write2image4xpu: " + str(origin2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2xpu ['ORIGIN'] = origin2xpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2xpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2xpu-trying-by-write2image4xpu: " + str(region2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2xpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2xpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2xpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2xpu-set-by-write2image4xpu to: " + str(region2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2xpu-failed-by-write2image4xpu: " + str(region2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2xpu ['REGION'] = region2xpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2xpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2xpu-trying-by-write2image4xpu: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2xpu = int(width2xpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2xpu-set-by-write2image4xpu to: " + str(rowpitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2xpu-failed-by-write2image4xpu: " + str(rowpitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2xpu ['ROWPITCH'] = rowpitch2xpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2xpu = None
            size2xpu = int(width2xpu * height2xpu)                                                                  # CALCULATED MEMORY
            if size2xpu == 0:
                size2xpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2xpu-trying-by-write2image4xpu: " + str(pointer2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2xpu = Arena4Xi(size2xpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2xpu-got-by-write2image4xpu: " + str(arena2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2xpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4xpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2xpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4xpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2xpu:
                    size2xpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2xpu-set-by-write2buffer4xpu to: " + str(size2xpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2xpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2xpu-set-by-write2buffer4xpu to: " + str(pointer2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2xpu-failed-by-write2image4xpu: " + str(pointer2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2xpu ['POINTER'] = pointer2xpu                            
            task4imagewrite2xpu ['SIZE']    = size2xpu                                                               # AUTOMATIC BY XPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2xpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2xpu-trying-by-write2image4xpu: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2xpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2xpu-set-by-write2image4xpu to: " + str(slicepitch2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2xpu-failed-by-write2image4xpu: " + str(slicepitch2xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2xpu ['SLICEPITCH'] = slicepitch2xpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2xpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4xpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2xpu-yielded-by-write2image4xpu: " + str(task4imagewrite2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4xpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2xpu = False
    channels2xpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2xpu-trying-by-write2image4xpu: " + str(channels2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2xpu, publisherchannel2xpu = task4imagewrite2xpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2xpu-got-by-write2image4xpu: " + str(consumerchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2xpu-got-by-write2image4xpu: " + str(publisherchannel2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2xpu = True
        
        control2writeimage4xpu['CONSUMERCHANNEL']  = consumerchannel2xpu
        
        
        control2writeimage4xpu['PUBLISHERCHANNEL'] = publisherchannel2xpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2xpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2xpu = False
    queue2xpu = None
    message2print = "{(Step 41A/70) trying" + "queue2xpu-trying-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2xpu = task4imagewrite2xpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2xpu-got-by-write2image4xpu: " + str(queue2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2xpu = True
        control2writeimage4xpu['QUEUE']  = queue2xpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2xpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4xpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4xpu-trying-by-write2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4xpu = DummyProcess4Xi(target=manoeuvre2readimage4xpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2xpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2xpu, publisherchannel2xpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4xpu-created-by-write2image4xpu: " + str(dummy2image4xpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4xpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4xpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4xpu-returning-by-write2image4xpu: " + str(control2writeimage4xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4xpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4xpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF XPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF XPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4xpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF XPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF XPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF XPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4xpu(task4release2xpu=None):

    '''
    releases xpu2image4xi
    '''
    global xpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4xpu"
    method2print = "release2image4xpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4xpu-started: " + str(release2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4xpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2xpu-to-release2image4xpu: " + str(task4release2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            xpu2image4xi = create2image4xpu()
        except:
            xpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; xpu2image4xi-yielded-by-release2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4xpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2xpu is None:
        task4release2xpu = {}
        task4release2xpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        xpu2image4xi['COUNT2REFERENCE'] = xpu2image4xi['COUNT2REFERENCE'] - task4release2xpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4xpu: " + str(xpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if xpu2image4xi['COUNT2REFERENCE'] < 0:
            xpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4xpu: " + " ** " + str(xpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4xpu : release2image4xpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4xpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            xpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4xpu: " + " ** " + str(xpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; xpu2image4xi-yielded-by-release2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    xpu2image4xi ['ACK2RETURN']  = code2ack
    xpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; xpu2image4xi-returning-by-release2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4xpu',
    #  'METHOD': 'create2image4xpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4xpu', 'METHOD': 'create2buffer4xpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return xpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4xpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF XPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4xpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF XPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF XPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4xpu(task4retain2xpu=None):

    '''
    retains either xpu2image4xi
    '''
    global xpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4xpu"
    method2print = "retain2image4xpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4xpu-started: " + str(retain2image4xpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4xpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2xpu-to-retain2image4xpu: " + str(task4retain2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2xpu is None:
        task4retain2xpu = {}
        task4retain2xpu['DIMENSION'] = 2
        task4retain2xpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2xpu-set-by-retain2image4xpu to: " + str(task4retain2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2xpu.keys():
        if task4retain2xpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2xpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4xpu to: " + str(task4retain2xpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2xpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2xpu.keys():
        pass
    else:
        task4retain2xpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4xpu to: " + str(task4retain2xpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2xpu['COUNT2REFERENCE'] < 1:
        task4retain2xpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4xpu to: " + str(task4retain2xpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if xpu2image4xi is None:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 4A/70) created" + "; xpu2image4xi-created-by-retain2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            xpu2image4xi = create2image4xpu()
            message2print = "{(Step 4B/70) created" + "; xpu2image4xi-created-by-retain2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            xpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; xpu2image4xi-yielded-by-retain2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        xpu2image4xi['COUNT2REFERENCE'] = xpu2image4xi['COUNT2REFERENCE'] + task4retain2xpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4xpu: " + str(xpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if xpu2image4xi['COUNT2REFERENCE'] <= 0:
            xpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4xpu: " + " ** " + str(xpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            xpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4xpu: " + " ** " + str(xpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; xpu2image4xi-yielded-by-retain2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    xpu2image4xi ['ACK2RETURN']  = code2ack
    xpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; xpu2image4xi-returning-by-retain2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4xpu', 'METHOD': 'create2image4xpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return xpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4xpu" usage = "
#   #1
#   print('(Step 3A/33) OF XPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR XPU', time4xi())  
#   try:
#       imageinfo2xpu = get2imageinfo4xpu()                                                                       
#       print('(Step 3B/33) OF XPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR XPU: ', imageinfo2xpu) 
#   except:
#       print('(Step 3Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR XPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF XPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF XPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4xpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF XPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF XPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF XPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4xpu(task4getinfo2image=None):

    '''
    gets xpu2image4xi
    '''
    
    global xpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4xpu"
    method2print = "get2imageinfo4xpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4xpu-started: " + str(get2imageinfo4xpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4xpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4xpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2xpu = {}
    try: 
        if xpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR XPU           
            # -------------------------------------------------------------------------------------           
            image2xpu = create2image4xpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2xpu-initialized-by-get2imageinfo4xpu to: " + str(image2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; xpu2image4xi-using-by-get2imageinfo4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4xpu', 
            #  'METHOD': 'create2image4xpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF XPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2xpu = xpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2xpu-got-by-get2imageinfo4xpu: " + str(image2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = xpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = xpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4xpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4xpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2xpu [key2task] = xpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2xpu-set-by-get2imageinfo4xpu to: " + str(image2xpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4xpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4xpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; xpu2image4xi-using-by-get2image4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2xpu = create2image4xpu()
            message2print = "{(Step 1F/33) initialized" + "; image2xpu-initialized-by-get2imageinfo4xpu to: " + str(image2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2xpu = xpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2xpu-got-by-get2imageinfo4xpu: " + str(image2xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4xpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2xpu [key2task] = xpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2xpu-set-by-get2imageinfo4xpu to: " + str(image2xpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4xpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2xpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2xpu-yielded-by-get2image4xpu: " + str(image2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2xpu-returning-by-get2imageinfo4xpu: " + str(image2xpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4xpu', 
    #  'METHOD': 'create2image4xpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4xpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4xpu', 
    #  'METHOD': 'create3image4xpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2xpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4xpu" usage = "
#
#   print('(Step 3A/33) OF XPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR XPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4xpu()                                                                       
#       print('(Step 3B/33) OF XPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR XPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR XPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4xpu(task4supportedimageformats2xpu=None):  

    '''
    gets supported image formats for xpu
    '''
    
    global xpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4xpu"
    method2print = "get2supportedimageformats4xpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4xpu-started: " + str(get2supportedimageformats4xpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4xpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2xpu-passed-to-get2supportedimageformats4xpu: " + str(task4supportedimageformats2xpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2xpu is None:
        task4supportedimageformats2xpu = {} 
        task4supportedimageformats2xpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2xpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2xpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2xpu = {} 
    try: 
        if xpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2xpu = create2image4xpu()                                                                          # create xpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2xpu-initialized-by-get2supportedimageformats4xpu to: " + str(image2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR XPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2xpu = xpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2xpu-got-by-get2supportedimageformats4xpu: " + str(supportedimageformats2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2xpu = create2image4xpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2xpu-initialized-by-get2supportedimageformats4xpu to: " + str(image2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR XPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2xpu = xpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2xpu-got-by-get2supportedimageformats4xpu: " + str(supportedimageformats2xpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2xpu-yielded-by-get2image4xpu: " + str(supportedimageformats2xpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2xpu-returning-by-get2supportedimageformats4xpu: " + str(supportedimageformats2xpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2xpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4xpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF XPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR XPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2xpu = wait2image4xpu()
#       keys2response = response4wait2xpu.keys()
#       print('(Step 16B/700) OF XPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR XPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2xpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4xpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4xpu', 'METHOD': 'create2image4xpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4xpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR XPU: ', wait2xpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4xpu(task4wait2image=None):

    '''
    waits xpu2image4xpu
    '''
    
    global xpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4xpu"
    method2print = "wait2image4xpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4xpu-started: " + str(wait2image4xpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4xpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if xpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            xpu2image4xi = create2image4xpu()
            
            message2print = "{(Step 1B/33) created" + "; xpu2image4xi-created-by-wait2image4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4xpu = None
            try:
                queue2image4xpu = xpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4xpu-got-by-wait2image4xpu to: " + str(queue2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; xpu2xi-yielded-by-wait2image4xpu: " + str(xpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4xpu = 0
            try:                    
                size2image4xpu = queue2image4xpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4xpu-got-by-wait2image4xpu to: " + str(size2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; xpu2xi-yielded-by-wait2image4xpu: " + str(xpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4xpu = True       
            try:                    
                empty2image4xpu= queue2image4xpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4xpu-got-by-wait2image4xpu to: " + str(empty2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4xpu-yielded-by-wait2image4xpu: " + str(empty2image4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4xpu = False       
            try:                                
                full2image4xpu = queue2image4xpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4xpu-got-by-wait2image4xpu to: " + str(full2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4xpu-yielded-by-wait2image4xpu: " + str(full2image4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; xpu2image4xi-finished-by-wait2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR XPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; xpu2image4xi-used-by-wait2image4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4xpu = None
            try:
                queue2image4xpu = xpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4xpu-got-by-wait2image4xpu to: " + str(queue2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; xpu2xi-yielded-by-wait2image4xpu: " + str(xpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4xpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4xpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4xpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4xpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4xpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4xpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4xpu-warned-by-wait2image4xpu: " + " ** " + str(queue2image4xpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4xpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4xpu = 0        
            try:                    
                size2queue4xpu = queue2image4xpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4xpu-got-by-wait2image4xpu to: " + str(size2queue4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                xpu2image4xi ['SIZE2QUEUE'] = size2queue4xpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4xpu to: " + str(xpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4xpu-yielded-by-wait2image4xpu: " + str(size2queue4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4xpu = False
            try:                    
                empty2queue4xpu = queue2image4xpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4xpu-got-by-wait2image4xpu to: " + str(empty2queue4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                xpu2image4xi ['EMPTY2QUEUE'] = empty2queue4xpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4xpu to: " + str(xpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4xpu-yielded-by-wait2image4xpu: " + str(empty2queue4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4xpu = False
            try:                                
                full2queue4xpu = queue2image4xpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4xpu-got-by-wait2image4xpu to: " + str(full2queue4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                xpu2image4xi ['FULL2QUEUE'] = full2queue4xpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4xpu to: " + str(xpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4xpu-yielded-by-wait2image4xpu: " + str(full2queue4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4xpu = None
            try:                                
                state2queue4xpu = queue2image4xpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4xpu-got-by-wait2image4xpu to: " + str(state2queue4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                xpu2image4xi ['STATE2QUEUE'] = state2queue4xpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4xpu to: " + str(xpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4xpu-yielded-by-wait2image4xpu: " + str(state2queue4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; xpu2image4xi-finished-by-wait2image4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            xpu2image4xpu = create2image4xpu()
            message2print = "{(Step 1R/33) got" + "; xpu2image4xi-got-by-wait2image4xpu to: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4xpu = None
            try:
                queue2image4xpu = xpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4xpu-got-by-wait2image4xpu to: " + str(queue2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4xpu-yielded-by-wait2image4xpu: " + str(queue2image4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4xpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4xpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4xpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4xpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4xpu-warned-by-wait2image4xpu: " + " ** " + str(queue2image4xpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4xpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4xpu = 0                             
            try:                    
                size2image4xpu = queue2image4xpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4xpu-got-by-wait2image4xpu to: " + str(size2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4xpu-yielded-by-wait2image4xpu: " + str(size2image4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4xpu = True          
            try:                    
                empty2image4xpu = queue2image4xpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4xpu-got-by-wait2image4xpu to: " + str(empty2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; xpu2xi-yielded-by-wait2image4xpu: " + str(xpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4xpu = False          
            try:                                
                full2image4xpu= queue2image4xpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4xpu-got-by-wait2image4xpu to: " + str(full2image4xpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4xpu-yielded-by-wait2image4xpu: " + str(full2image4xpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; xpu2image4xi-finished-by-wait2image4xpu: " + str(xpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            xpu2image4xpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; xpu2image4xi-yielded-by-wait2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; xpu2image4xi-returning-by-wait2image4xpu: " + str(xpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return xpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "XPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for XPU 2D images" usage = "2D Image4Xi of XPU"
#>

# =================================================================================================
class XPU2Image4Xi(object): 
    ''' 
    XPU2Image4Xi offers methods for XPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2xpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-XPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF XPU2IMAGE4XI5TEST - CREATING IMAGE4XPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2xpu = XPU2Image4Xi()
    #       print('(Step 5B/33) OF XPU2IMAGE4XI5TEST - CREATED IMAGE4XPU OBJECT: ', imageobject2xpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4XPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2xpu = None):
        '''
        _init__ method initializes XPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "XPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-XPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; xpu2image2xi-of-XPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-xpu2image2xi-of-XPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2xpu-sharing-by-xpu2image2xi-of-XPU2Image4Xi: " + str(XPU2Image4Xi.classvariable2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2xpu-passed-to-init-of-XPU2Image4Xi: " + str(task4imagecreate2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2xpu-passing-to-create2image4xpu-by-init-of-XPU2Image4Xi: " + str(task4imagecreate2xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        xpu2image2xi = create2image4xpu(task4imagecreate2xpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4xpu-performed-by-init-of-XPU2Image4Xi: " + str(create2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4xpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; xpu2image2xi-of-XPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { XPU2Image4Xi : __init__ ; (Step 3A/33) activated; xpu2image2xi-of-XPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-xpu2image2xi-of-XPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2xpu-shared-by-xpu2image2xi-of-XPU2Image4Xi: " + str(XPU2Image4Xi.classvariable2xpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-XPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF XPU2IMAGE4XI5TEST - TRYING TO CREATE XPU IMAGE', time4xi()) 
    #   try:
    #       xi2xpu = XPU2Image4Xi()                            
    #       image2xpu = xi2xpu.create2image()                                  
    #       print('(Step 23B/33) OF XPU2IMAGE4XI5TEST - CREATED XPU IMAGE: ', image2xpu)
    #   except:
    #       print('(Step 23Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO CREATE XPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of XPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-XPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4xpu-by-create2image-of-XPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        xpu2image2xi = create2image4xpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4xpu-performed-by-create2image-of-XPU2Image4Xi: " + str(create2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4xpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; xpu2image2xi-returning-by-create2image-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4xpu',
        #  'METHOD': 'create2image4xpu'},
        #  'WIDTH': 800 ...

        return xpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF XPU2IMAGE4XI - TRYING TO SET XPU IMAGE')
    #   try:
    #       cc2xi = XPU2Image4Xi()                            
    #       image2xpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF XPU4XI5TEST - SET XPU IMAGE: ', image2xpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF XPU4XI5TEST *** FAILED TO SET XPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image XPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-XPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4xpu-by-set2image-of-XPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        xpu2image2xi = set2image4xpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4xpu-performed-by-set2image-of-XPU2Image4Xi: " + str(set2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4xpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; xpu2image2xi-returning-by-set2image-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4xpu', 'METHOD': 'create2image4xpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4xpu',
        #  'METHOD': 'create2buffer4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return xpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-XPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF XPU2IMAGE4XI - TRYING TO GET XPU 2D IMAGE')
    #   try:
    #       cc2xi = XPU2Image4Xi()                            
    #       image2xpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF XPU4XI5TEST - GOT XPU IMAGE: ', image2xpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF XPU4XI5TEST *** FAILED TO GET XPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of XPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        xpu2image2xi = get2imageinfo4xpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; xpu2image2xi-returning-by-get2imageinfo-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4xpu', 'METHOD': 'create2image4xpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4xpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4xpu',
        #  'METHOD': 'create2buffer4xpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4xpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return xpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-XPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF XPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR XPU')
    #  try:
    #      cc2xi = XPU2Image4Xi()                            
    #      supportedimageformats2xpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF XPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR XPU: ', supportedimageformats2xpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF XPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR XPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of XPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2xpu = get2supportedimageformats4xpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2xpu-returning-by-get2imageinfo-of-XPU2Image4Xi: " + str(supportedimageformats2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2xpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2xpu = {}
    #   try:
    #       xi2xpu = XPU2Image4Xi()    
    #       response4read2xpu = xi2xpu.read2image()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2XPU: ', response4read2xpu) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-XPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4xpu-by-read2image-of-XPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2xpu = read2image4xpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4xpu-performed-by-read2image-of-XPU2Image4Xi: " + str(read2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4xpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2xpu-returning-by-read2image-of-XPU2Image4Xi: " + str(response4readimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2xpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2xpu = XPU2Image4Xi() 
    #   ...
    #   response4map2xpu = {}
    #   try:
    #       response4map2xpu = xi2xpu.map2image()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2XPU: ', response4map2xpu) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-XPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4xpu-by-map2image-of-XPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2xpu = map2image4xpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4xpu-performed-by-map2image-of-XPU2Image4Xi: " + str(map2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4xpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2xpu-returning-by-map2image-of-XPU2Image4Xi: " + str(response4mapimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2xpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-XPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF XPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2xpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF XPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF XPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF XPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF XPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2xpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF XPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF XPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-XPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4xpu-by-release2image-of-XPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        xpu2image2xi = release2image4xpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4xpu-performed-by-release2image-of-XPU2Image4Xi: " + str(release2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; xpu2image2xi-returning-by-release2image-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return xpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-XPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF XPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF XPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2xpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF XPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF XPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF XPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2xpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF XPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF XPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF XPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-XPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4xpu-by-retain-of-XPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        xpu2image2xi = retain2image4xpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4xpu-performed-by-retain2image-of-XPU2Image4Xi: " + str(retain2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4xpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; xpu2image2xi-returning-by-retain2image-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4xpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4xpu', 
        #  'METHOD': 'create2image4xpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return xpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-XPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF XPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR XPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2xpu = {}
    #   try:
    #       response4wait2Dimage = xi2xpu.wait2image()
    #       print('(Step 122B/700) OF XPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR XPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR XPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-XPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-XPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4xpu-by-wait2image-of-XPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        xpu2image2xi = wait2image4xpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4xpu-performed-by-wait2image-of-XPU2Image4Xi: " + str(wait2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4xpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; xpu2image2xi-returning-by-wait2image-of-XPU2Image4Xi: " + str(xpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return xpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2xpu = {}
    #   try:
    #       response4write2xpu = xi2xpu.write2image()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2XPU: ', response4write2xpu) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-XPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4xpu-by-write2image-of-XPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2xpu = write2image4xpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4xpu-performed-by-write2image-of-XPU2Image4Xi: " + str(write2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4xpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2xpu-returning-by-write2image-of-XPU2Image4Xi: " + str(response4writeimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2xpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2xpu = {}
    #   try:
    #       response4copy2xpu = xi2xpu.copy2image()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2XPU: ', response4copy2xpu) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-XPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4xpu-by-copy2image-of-XPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2xpu = copy2image4xpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4xpu-performed-by-copy2image-of-XPU2Image4Xi: " + str(copy2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4xpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2xpu-returning-by-copy2image-of-XPU2Image4Xi: " + str(response4copyimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2xpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2xpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2XPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-XPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4xpu-by-copy2imagetobuffer-of-XPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2xpu = copy2imagetobuffer4xpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4xpu-performed-by-copy2imagetobuffer-of-XPU2Image4Xi: " + str(copy2imagetobuffer4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4xpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2xpu-returning-by-copy2imagetobuffer-of-XPU2Image4Xi: " + str(response4copyimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2xpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-XPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF XPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2xpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF XPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2XPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF XPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4xpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-XPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4xpu-by-copy2buffertoimage-of-XPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2xpu = copy2buffertoimage4xpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4xpu-performed-by-copy2buffertoimage-of-XPU2Image4Xi: " + str(copy2buffertoimage4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4xpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2xpu-returning-by-copy2buffertoimage-of-XPU2Image4Xi: " + str(response4copyimage2xpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2xpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-XPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF XPU2IMAGE4XI5TEST - TRYING TO TRACE XPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2xpu = None
    #   try:
    #       trace2xpu = trace2image4xpu() 
    #       print('(Step 15B/25) OF XPU2IMAGE4XI5TEST - TRACE2XPU: ', trace2xpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4xpu', 'METHOD': 'trace2image4xpu'}
    #    except:
    #       print('(Step 15Z1/25) OF XPU2IMAGE4XI5TEST *** FAILED TO TRACE XPU: ', trace2xpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-XPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4xpu-by-trace2image-of-XPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4xpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4xpu-performed-by-trace2image-of-XPU2Image4Xi: " + str(trace2image4xpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4xpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-XPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4xpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4xpu',
        #  'METHOD': 'trace2image4xpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-XPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF XPU2IMAGE4XI - TRYING TO SET XPU IMAGE')
    #   try:
    #       cc2xi = XPU2Image4Xi()                            
    #       image2xpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF XPU4XI5TEST - SET XPU IMAGE: ', image2xpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF XPU4XI5TEST *** FAILED TO SET XPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of XPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "XPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-XPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-XPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2xpu4image = {} 
        try:
            callback2xpu4image = set2imagecallback4xpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2xpu4image" + "; callback2xpu4image-set-by-set2imagecallback-of-XPU2Image4Xi to: " + str(callback2xpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4xpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2xpu4image-failed-by-set2imagecallback-of-XPU2Image4Xi: " + str(callback2xpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2xpu4image-returning-by-set2imagecallback-of-XPU2Image4Xi: " + str(callback2xpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4xpu at 0x0000000003AB31C8> 
                                             
        return callback2xpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-XPU2Image4Xi" usage = "
    #   try:
    #       print(xi2xpu.sense())
    #       print('(Step 24B/33) OF XPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses XPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'XPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; XPU2Image4Xi-sensing-by-sense-of-XPU2Image4Xi: " + str(XPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'xpu2image4xi.XPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-XPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2xpu-of-XPU2Image4Xi'] = XPU2Image4Xi.classvariable2xpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-XPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; XPU2Image4Xi-sensed-by-sense-of-XPU2Image4Xi: " + str(XPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'xpu2image4xi.XPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-XPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>, 'classvariable2xpu-of-XPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-XPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF XPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2xpu.reason())
    #       print('(Step 25B/33) OF XPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons XPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'XPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; XPU2Image4Xi-reasoning-by-reason-of-XPU2Image4Xi: " + str(XPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'xpu2image4xi.XPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-XPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-XPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-XPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; XPU2Image4Xi-reasoned-by-reason-of-XPU2Image4Xi: " + str(XPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'xpu2image4xi.XPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-XPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-XPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-XPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF XPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2xpu.ace())
    #       print('(Step 26B/33) OF XPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF XPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces XPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'XPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-XPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; XPU2Image4Xi-acing-by-ace-of-XPU2Image4Xi: " + str(XPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'xpu2image4xi.XPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-XPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-XPU2Image4Xi'] = XPU2Image4Xi.classvariable2xpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-XPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; XPU2Image4Xi-aced-by-ace-of-XPU2Image4Xi: " + str(XPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'xpu2image4xi.XPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-XPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-XPU2Image4Xi': 4, 'object8xi': <xpu2image4xi.XPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   xpu2image4xi5test.txt
#   sense-xpu2image4xi5test.xml
#   reason-xpu2image4xi5test.xml
#   ace-xpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of xpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
